# -*- coding: utf-8 -*-
from __future__ import print_function
import collections
import itertools
from abjad.tools import patterntools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import select


class TieSpecifier(AbjadValueObject):
    r'''Tie specifier.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_strip_ties',
        '_tie_across_divisions',
        '_tie_consecutive_notes',
        '_use_messiaen_style_ties',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        strip_ties=None,
        tie_across_divisions=None,
        tie_consecutive_notes=None,
        use_messiaen_style_ties=None,
        ):
        if strip_ties is not None:
            strip_ties = bool(strip_ties)
        self._strip_ties = strip_ties
        prototype = (
            type(None),
            bool,
            collections.Sequence,
            patterntools.Pattern,
            patterntools.PatternList,
            )
        assert isinstance(tie_across_divisions, prototype)
        self._tie_across_divisions = tie_across_divisions
        if tie_consecutive_notes is not None:
            tie_consecutive_notes = bool(tie_consecutive_notes)
        self._tie_consecutive_notes = tie_consecutive_notes
        if self.tie_consecutive_notes and self.strip_ties:
            message = 'can not tie leaves and strip ties at same time.'
            raise Exception(message)
        if use_messiaen_style_ties is not None:
            use_messiaen_style_ties = bool(use_messiaen_style_ties)
        self._use_messiaen_style_ties = use_messiaen_style_ties

    ### SPECIAL METHODS ###

    def __call__(self, divisions):
        r'''Calls tie specifier on `divisions`.

        Returns none.
        '''
        self._do_tie_across_divisions(divisions)
        self._do_tie_consecutive_notes(divisions)
        self._do_strip_ties(divisions)
        self._configure_messiaen_style_ties(divisions)

    ### PRIVATE METHODS ###

    def _configure_messiaen_style_ties(self, divisions):
        if not self.use_messiaen_style_ties:
            return
        tie_spanners = set()
        for leaf in iterate(divisions).by_leaf():
            tie_spanners_ = inspect_(leaf).get_spanners(
                prototype=spannertools.Tie,
                in_parentage=True,
                )
            tie_spanners.update(tie_spanners_)
        for tie_spanner in tie_spanners:
            tie_spanner._use_messiaen_style_ties = True

    def _do_strip_ties(self, divisions):
        if not self.strip_ties:
            return
        for division in divisions:
            for leaf in iterate(division).by_leaf():
                detach(spannertools.Tie, leaf)

    def _do_tie_across_divisions(self, divisions):
        if not self.tie_across_divisions:
            return
        if self.strip_ties:
            return
        if self.tie_consecutive_notes:
            return
        length = len(divisions)
        tie_across_divisions = self.tie_across_divisions
        if isinstance(tie_across_divisions, bool):
            tie_across_divisions = [tie_across_divisions]
        if not isinstance(tie_across_divisions, patterntools.Pattern):
            tie_across_divisions = patterntools.Pattern.from_vector(
                tie_across_divisions)
        pairs = sequencetools.Sequence(divisions).nwise()
        rest_prototype = (scoretools.Rest, scoretools.MultimeasureRest)
        for i, pair in enumerate(pairs):
            if not tie_across_divisions.matches_index(i, length):
                continue
            division_one, division_two = pair
            leaf_one = next(iterate(division_one).by_leaf(reverse=True))
            leaf_two = next(iterate(division_two).by_leaf())
            leaves = [leaf_one, leaf_two]
            if isinstance(leaf_one, rest_prototype):
                continue
            if isinstance(leaf_two, rest_prototype):
                continue
            prototype = (scoretools.Note, scoretools.Chord)
            if not all(isinstance(x, prototype) for x in leaves):
                continue
            logical_tie_one = inspect_(leaf_one).get_logical_tie()
            logical_tie_two = inspect_(leaf_two).get_logical_tie()
            for tie in inspect_(leaf_one).get_spanners(spannertools.Tie):
                detach(tie, leaf_one)
            for tie in inspect_(leaf_two).get_spanners(spannertools.Tie):
                detach(tie, leaf_two)
            combined_logical_tie = logical_tie_one + logical_tie_two
            tie_spanner = spannertools.Tie(
                use_messiaen_style_ties=self.use_messiaen_style_ties,
                )
            tie_spanner._unconstrain_contiguity()
            if tie_spanner._attachment_test_all(combined_logical_tie):
                attach(tie_spanner, combined_logical_tie)
            tie_spanner._constrain_contiguity()

    def _do_tie_consecutive_notes(self, divisions):
        if not self.tie_consecutive_notes:
            return
        leaves = select(divisions).by_leaf()
        for leaf in leaves:
            detach(spannertools.Tie, leaf)
        pairs = itertools.groupby(leaves, lambda _: _.__class__)

        def _get_pitches(component):
            if isinstance(component, scoretools.Note):
                return component.written_pitch
            elif isinstance(component, scoretools.Chord):
                return component.written_pitches
            else:
                raise TypeError(component)
        for class_, group in pairs:
            group = list(group)
            if not isinstance(group[0], (scoretools.Note, scoretools.Chord)):
                continue
            subpairs = itertools.groupby(group, lambda _: _get_pitches(_))
            for pitches, subgroup in subpairs:
                subgroup = list(subgroup)
                if len(subgroup) == 1:
                    continue
                tie = spannertools.Tie()
                assert tie._attachment_test_all(subgroup)
                attach(tie, subgroup)

    ### PUBLIC PROPERTIES ###

    @property
    def strip_ties(self):
        r'''Is true when rhythm-maker should strip all ties from all leaves in
        each division.

        Set to true, false or none.
        '''
        return self._strip_ties

    @property
    def tie_across_divisions(self):
        r'''Is true or is a boolean vector when rhythm-maker should tie across
        divisons. Otherwise false.

        Set to true, false or to a boolean vector.
        '''
        return self._tie_across_divisions

    @property
    def tie_consecutive_notes(self):
        r'''Is true when rhythm-maker should tie consecutive notes.
        Otherwise false.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._tie_consecutive_notes

    @property
    def use_messiaen_style_ties(self):
        r'''Is true when ties should be Messiaen-style with the LilyPond
        ``\repeatTie`` command. Otherwise false.

        Set to true, false or none.
        '''
        return self._use_messiaen_style_ties



# -*- coding: utf-8 -*-
import abjad
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools.topleveltools import inspect_


def make_desordre_measure(pitches):
    '''Makes a measure composed of *Désordre cells*.

    `pitches` is a list of lists of number (e.g., [[1, 2, 3], [2, 3, 4]])

    The function returns a measure.
    '''

    for sequence in pitches:
        container = abjad.demos.desordre.make_desordre_cell(sequence)
        time_signature = inspect_(container).get_duration()
        time_signature = mathtools.NonreducedFraction(time_signature)
        time_signature = time_signature.with_denominator(8)
        measure = scoretools.Measure(time_signature, [container])

    return measure


# -*- coding: utf-8 -*-


def timespan_2_starts_during_timespan_1(
    timespan_1=None,
    timespan_2=None,
    hold=False,
    ):
    r'''Makes time relation indicating that `timespan_2` starts
    during `timespan_1`.

    ::

        >>> relation = timespantools.timespan_2_starts_during_timespan_1()
        >>> print(format(relation))
        timespantools.TimespanTimespanTimeRelation(
            inequality=timespantools.CompoundInequality(
                [
                    timespantools.Inequality('timespan_1.start_offset <= timespan_2.start_offset'),
                    timespantools.Inequality('timespan_2.start_offset < timespan_1.stop_offset'),
                    ],
                logical_operator='and',
                ),
            )

    ..  container:: example

        ::

            >>> staff_1 = Staff("c'4 d'4 e'4 f'4 g'2 c''2")
            >>> staff_2 = Staff("c'2 b'2 a'2 g'2")
            >>> score = Score([staff_1, staff_2])
            >>> show(score) # doctest: +SKIP

        ::

            >>> start_offsets = [inspect_(note).get_timespan().start_offset for note in staff_1]
            >>> stop_offsets = [inspect_(note).get_timespan().stop_offset for note in staff_1]

        ::

            >>> timespan_1 = timespantools.Timespan(Offset(1, 4), Offset(5, 4))
            >>> time_relation = \
            ...     timespantools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)
            >>> start_index, stop_index = time_relation.get_offset_indices(
            ...     start_offsets, stop_offsets)

        ::

            >>> selected_notes = staff_1[start_index:stop_index]
            >>> selected_notes
            Selection([Note("d'4"), Note("e'4"), Note("f'4"), Note("g'2")])

        ::

            >>> label(selected_notes).color_leaves('red')

        ::

            >>> show(score) # doctest: +SKIP

    Returns time relation or boolean.
    '''
    from abjad.tools import timespantools

    inequality = timespantools.CompoundInequality([
        'timespan_1.start_offset <= timespan_2.start_offset',
        'timespan_2.start_offset < timespan_1.stop_offset',
        ])

    time_relation = timespantools.TimespanTimespanTimeRelation(
        inequality,
        timespan_1=timespan_1,
        timespan_2=timespan_2,
        )

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_has_indicator_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    attach('foo', staff)

    assert inspect_(staff).has_indicator(str)
    assert not inspect_(staff[0]).has_indicator(str)
    assert not inspect_(staff[1]).has_indicator(str)
    assert not inspect_(staff[2]).has_indicator(str)
    assert not inspect_(staff[3]).has_indicator(str)


def test_agenttools_InspectionAgent_has_indicator_02():

    staff = Staff("c'2 d'2")
    annotation = indicatortools.Annotation('name', 'value')
    attach(annotation, staff[0])

    assert inspect_(staff[0]).has_indicator(indicatortools.Annotation)
    assert not inspect_(staff[1]).has_indicator(indicatortools.Annotation)


def test_agenttools_InspectionAgent_has_indicator_03():

    staff = Staff("c'2 d'2")
    articulation = Articulation('staccato')
    attach(articulation, staff[0])

    assert inspect_(staff[0]).has_indicator(Articulation)
    assert not inspect_(staff[1]).has_indicator(Articulation)


def test_agenttools_InspectionAgent_has_indicator_04():

    staff = Staff("c'8 d'8 e'8 f'8")
    command = indicatortools.LilyPondCommand('break', 'closing')
    attach(command, staff[-1])

    assert not inspect_(staff[0]).has_indicator(indicatortools.LilyPondCommand)
    assert not inspect_(staff[1]).has_indicator(indicatortools.LilyPondCommand)
    assert not inspect_(staff[2]).has_indicator(indicatortools.LilyPondCommand)
    assert     inspect_(staff[3]).has_indicator(indicatortools.LilyPondCommand)


def test_agenttools_InspectionAgent_has_indicator_05():

    staff = Staff("c'2 d'2")
    comment = indicatortools.LilyPondComment('comment')
    attach(comment, staff[0])

    assert inspect_(staff[0]).has_indicator(indicatortools.LilyPondComment)
    assert not inspect_(staff[1]).has_indicator(indicatortools.LilyPondComment)


def test_agenttools_InspectionAgent_has_indicator_06():

    staff = Staff("c'2 d'2")
    stem_tremolo = indicatortools.StemTremolo(16)
    attach(stem_tremolo, staff[0])

    assert inspect_(staff[0]).has_indicator(indicatortools.StemTremolo)
    assert not inspect_(staff[1]).has_indicator(indicatortools.StemTremolo)


def test_agenttools_InspectionAgent_has_indicator_07():

    staff = Staff("c'8 d'8 e'8 f'8")
    time_signature = TimeSignature((4, 8))
    attach(time_signature, staff[0])

    assert inspect_(staff[0]).has_indicator(TimeSignature)
    assert not inspect_(staff[1]).has_indicator()
    assert not inspect_(staff[2]).has_indicator()
    assert not inspect_(staff[3]).has_indicator()
    assert not inspect_(staff).has_indicator()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_spannertools_Beam___init___01():

    staff = Staff("c'8 d'8 e'8 f'8 g'2")
    beam = Beam()
    attach(beam, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
            g'2
        }
        '''
        )

    assert len(beam) == 4


def test_spannertools_Beam___init___02():
    r'''Nonempty container.
    '''

    container = Container("c'8 c'8 c'8 c'8 c'8 c'8 c'8 c'8")
    beam = Beam()
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 [
            c'8
            c'8
            c'8
            c'8
            c'8
            c'8
            c'8 ]
        }
        '''
        )

    assert len(beam) == 8


def test_spannertools_Beam___init___03():
    r'''Nested nonempty containers.
    '''

    staff = Staff("{ c'8 c'8 c'8 c'8 } { c'8 c'8 c'8 c'8 }")
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                c'8
                c'8
                c'8
            }
            {
                c'8
                c'8
                c'8
                c'8 ]
            }
        }
        '''
        )

    assert len(beam) == 8


def test_spannertools_Beam___init___04():
    r'''Beamed container and top-level leaves housed in staff.
    '''

    staff = Staff("{ c'8 c'8 c'8 c'8 } c'8 c'8")
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                c'8
                c'8
                c'8
            }
            c'8
            c'8 ]
        }
        '''
        )

    assert len(beam) == 6


def test_spannertools_Beam___init___05():
    r'''Beamed leaves housed in staff and container.
    '''

    staff = Staff("{ c'8 c'8 c'8 c'8 } c'8 c'8")
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                c'8
                c'8
                c'8
            }
            c'8
            c'8 ]
        }
        '''
        )

    assert len(beam) == 6


def test_spannertools_Beam___init___06():
    r'''Staff with empty containers at the edges.
    '''

    staff = Staff(Container([]) * 2)
    staff.insert(1, Container(Note(0, (1, 8)) * 4))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
            }
            {
                c'8 [
                c'8
                c'8
                c'8 ]
            }
            {
            }
        }
        '''
        )

    assert len(beam) == 4


def test_spannertools_Beam___init___07():
    r'''Deeply nested containers of equal depth.
    '''

    voice = Voice("{ { c'8 cs'8 d'8 ef'8 } } { { e'8 f'8 fs'8 g'8 } }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                {
                    c'8 [
                    cs'8
                    d'8
                    ef'8
                }
            }
            {
                {
                    e'8
                    f'8
                    fs'8
                    g'8 ]
                }
            }
        }
        '''
        )

    assert len(beam) == 8


def test_spannertools_Beam___init___08():
    r'''Deeply nested containers of unequal depth.
    '''

    voice = Voice("{ { { c'8 cs'8 d'8 ef'8 } } } { e'8 f'8 fs'8 g'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                {
                    {
                        c'8 [
                        cs'8
                        d'8
                        ef'8
                    }
                }
            }
            {
                e'8
                f'8
                fs'8
                g'8 ]
            }
        }
        '''
        ), repr(f(voice))

    assert len(beam) == 8


def test_spannertools_Beam___init___09():
    r'''Voice with containers and top-level leaves.
    '''

    voice = Voice("{ c'8 cs'8 } d'8 { ef'8 e'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [
                cs'8
            }
            d'8
            {
                ef'8
                e'8 ]
            }
        }
        '''
        )

    assert len(beam) == 5

def test_spannertools_Beam___init___10():
    r'''Voice with tuplets and top-level leaves.
    '''

    voice = Voice(r"\times 2/3 { c'8 cs' d' } ef'8 \times 2/3 { e'8 f' fs' }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                cs'8
                d'8
            }
            ef'8
            \times 2/3 {
                e'8
                f'8
                fs'8 ]
            }
        }
        '''
        )

    assert len(beam) == 7


def test_spannertools_Beam___init___11():
    r'''Nested tuplets.
    '''

    tuplet = Tuplet((2, 3), r"c'4 \times 2/3 { c'8 c'8 c'8 } c'4")
    beam = Beam()
    attach(beam, tuplet[1][:])

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'4
            \times 2/3 {
                c'8 [
                c'8
                c'8 ]
            }
            c'4
        }
        '''
        )

    assert len(beam) == 3


def test_spannertools_Beam___init___12():
    r'''Beams can not cross voice boundaries.
    '''

    staff = Staff([
        Voice("c'8 cs'8 d'8"),
        Note("ef'8"),
        Voice("e'8 f' fs' g'")]
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                c'8
                cs'8
                d'8
            }
            ef'8
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    beam = Beam()
    leaves = select(staff).by_leaf()
    statement = 'attach(beam, leavs)'
    assert pytest.raises(Exception, statement)


def test_spannertools_Beam___init___13():
    r'''You can span the counttime components of like-named voices.
    '''

    staff = Staff([Voice("c'8 cs'8 d'8 ef'8"), Voice("e'8 f'8 fs'8 g'8")])
    staff[0].name = 'foo'
    staff[1].name = 'foo'
    beam = Beam()
    attach(beam, staff[0][:] + staff[1][:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "foo" {
                c'8 [
                cs'8
                d'8
                ef'8
            }
            \context Voice = "foo" {
                e'8
                f'8
                fs'8
                g'8 ]
            }
        }
        '''
        )

    assert len(beam) == 8


def test_spannertools_Beam___init___14():
    '''Like-named containers need not be lexically contiguous.
    '''

    container = Container(r'''
        <<
            \context Voice = "first" {
                c'8
                cs'8
                d'8
                ef'8
            }
            \context Voice = "second" {
                e'8
                f'8
                fs'8
                g'8
            }
        >>
        <<
            \context Voice = "second" {
                af'8
                a'8
                bf'8
                b'8
            }
            \context Voice = "first" {
                c''8
                cs''8
                d''8
                ef''8
            }
        >>
        ''')

    beam = Beam()
    attach(beam, container[0][0][:] + container[1][1][:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            <<
                \context Voice = "first" {
                    c'8 [
                    cs'8
                    d'8
                    ef'8
                }
                \context Voice = "second" {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
            >>
            <<
                \context Voice = "second" {
                    af'8
                    a'8
                    bf'8
                    b'8
                }
                \context Voice = "first" {
                    c''8
                    cs''8
                    d''8
                    ef''8 ]
                }
            >>
        }
        '''
        )

    assert len(beam) == 8


def test_spannertools_Beam___init___15():
    '''Asymmetric structures are no problem.
    '''

    container = Container(
        r'''
        <<
            \context Voice = "first" {
                c'8
                cs'8
                d'8
                ef'8
            }
            \context Voice = "second" {
                e'8
                f'8
                fs'8
                g'8
            }
        >>
        <<
            \context Voice = "first" {
                af'8
                a'8
                bf'8
                b'8
            }
        >>
        ''')

    beam = Beam()
    attach(beam, container[0][0][:] + container[1][0][:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            <<
                \context Voice = "first" {
                    c'8 [
                    cs'8
                    d'8
                    ef'8
                }
                \context Voice = "second" {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
            >>
            <<
                \context Voice = "first" {
                    af'8
                    a'8
                    bf'8
                    b'8 ]
                }
            >>
        }
        '''
        )

    assert len(beam) == 8


def test_spannertools_Beam___init___16():
    r'''Notes in voice accept spanner even lodged within
    simultaneous parent container.
    '''

    container = Container(
        r'''
        <<
            \new Voice {
                c'8
                cs'8
                d'8
                ef'8
            }
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        >>
        ''')

    beam = Beam()
    attach(beam, container[0][:])

    assert format(container) == stringtools.normalize(
        r'''
        <<
            \new Voice {
                c'8 [
                cs'8
                d'8
                ef'8 ]
            }
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        >>
        '''
        )

    assert len(beam) == 4


def test_spannertools_Beam___init___17():
    r'''You can not yet span noncontiguous counttime components
    in the same logical voice. Lilypond is happy with this situation, though.
    '''

    staff = Staff(
        r'''
        c'8
        cs'8
        <<
            \new Voice {
                d'8
                ef'8
                e'8
                f'8
            }
            \new Voice {
                fs'8
                g'8
                af'8
                a'8
            }
        >>
        bf'8
        b'8
        ''')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            <<
                \new Voice {
                    d'8
                    ef'8
                    e'8
                    f'8
                }
                \new Voice {
                    fs'8
                    g'8
                    af'8
                    a'8
                }
            >>
            bf'8
            b'8
        }
        '''
        )

    leaves = staff[:2] + staff[-2:]
    beam = Beam()
    statement = 'beam.attacch(leaves)'
    assert pytest.raises(Exception, statement)


def test_spannertools_Beam___init___18():
    r'''You can span counttime components in three chunks.
    '''

    staff = Staff(r'''
        \context Voice = "foo" {
            c'8
            cs'8
            d'8
            ef'8
        }
        <<
            \context Voice = "foo" {
                e'8
                f'8
                fs'8
                g'8
            }
            \context Voice = "bar" {
                af'8
                a'8
                bf'8
                b'8
            }
        >>
        \context Voice = "foo" {
            c''8
            cs''8
            d''8
            ef''8
        }
        ''')

    leaves = staff[0][:] + staff[1][0][:] + staff[2][:]
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "foo" {
                c'8 [
                cs'8
                d'8
                ef'8
            }
            <<
                \context Voice = "foo" {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
                \context Voice = "bar" {
                    af'8
                    a'8
                    bf'8
                    b'8
                }
            >>
            \context Voice = "foo" {
                c''8
                cs''8
                d''8
                ef''8 ]
            }
        }
        '''
        )

    assert len(beam) == 12


def test_spannertools_Beam___init___19():
    r'''You can not span across differently named voices.
    '''

    staff = Staff([Voice("c'8 cs'8 d'8 ef'8"), Voice("e'8 f'8 fs'8 g'8")])
    staff[0].name = 'foo'
    staff[1].name = 'bar'

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "foo" {
                c'8
                cs'8
                d'8
                ef'8
            }
            \context Voice = "bar" {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(staff)
    beam = Beam()
    statement = 'attach(beam, leaves)'
    assert pytest.raises(Exception, statement)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSegment___max___01():

    pitch_segment = pitchtools.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])
    assert max(pitch_segment) == NamedPitch(7)



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo_is_imprecise_01( ):
    r'''Tempo is imprecise if either duration or units_per_minute is none,
    or if units_per_minute is a tuple representing a tempo range.
    '''

    assert not Tempo(Duration(1, 4), 60).is_imprecise
    assert not Tempo(4, 60, 'Langsam').is_imprecise
    assert Tempo(textual_indication='Langsam').is_imprecise
    assert Tempo(4, (35, 50), 'Langsam').is_imprecise
    assert Tempo(Duration(1, 4), (35, 50)).is_imprecise



# -*- coding: utf-8 -*-
from abjad import *


def test_stringtools_is_snake_case_file_name_01():

    assert stringtools.is_snake_case_file_name('foo_bar.blah')
    assert stringtools.is_snake_case_file_name('foo.blah')
    assert stringtools.is_snake_case_file_name('foo_bar')
    assert stringtools.is_snake_case_file_name('foo')


def test_stringtools_is_snake_case_file_name_02():

    assert stringtools.is_snake_case_file_name('')


def test_stringtools_is_snake_case_file_name_03():

    assert not stringtools.is_snake_case_file_name('foo_bar.')
    assert not stringtools.is_snake_case_file_name('Foo_bar.blah')
    assert not stringtools.is_snake_case_file_name('foo_bar.Blah')
    assert not stringtools.is_snake_case_file_name('Foo_bar')



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Staff_time_signature_01():
    r'''Force time signature on nonempty staff.
    '''

    staff = Staff(Note("c'4") * 8)
    time_signature = TimeSignature((2, 4))
    attach(time_signature, staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \time 2/4
            c'4
            c'4
            c'4
            c'4
            c'4
            c'4
            c'4
            c'4
        }
        '''
        )


def test_scoretools_Staff_time_signature_02():
    r'''Force time signature on empty staff.
    '''

    staff = Staff([])
    time_signature = TimeSignature((2, 4))
    attach(time_signature, staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \time 2/4
        }
        '''
        )


def test_scoretools_Staff_time_signature_03():
    r'''Staff time signature carries over to staff-contained leaves.
    '''

    staff = Staff(Note("c'4") * 8)
    time_signature = TimeSignature((2, 4))
    attach(time_signature, staff)
    for x in staff:
        assert inspect_(x).get_effective(TimeSignature) \
            == TimeSignature((2, 4))


def test_scoretools_Staff_time_signature_04():
    r'''Staff time signature set and then clear.
    '''

    staff = Staff(Note("c'4") * 8)
    time_signature = TimeSignature((2, 4))
    attach(time_signature, staff)
    detach(time_signature, staff)
    for leaf in staff:
        assert inspect_(leaf).get_effective(
            TimeSignature) is None



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure_time_signature_update_01():
    r'''Measures allow time signature update.
    '''

    measure = Measure((4, 8), "c'8 d'8 e'8 f'8")
    measure.pop()
    detach(TimeSignature, measure)
    time_signature = TimeSignature((3, 8))
    attach(time_signature, measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/8
            c'8
            d'8
            e'8
        }
        '''
        )

    assert inspect_(measure).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSegment___init___01():
    r'''Initialize named pitch segment from named pitches.
    '''

    named_pitches = []
    named_pitches.append(NamedPitch("c''"))
    named_pitches.append(NamedPitch("cs''"))
    named_pitches.append(NamedPitch("d''"))
    named_pitches.append(NamedPitch("ds''"))
    named_pitch_segment = pitchtools.PitchSegment(
        named_pitches, item_class=NamedPitch)

    assert isinstance(named_pitch_segment, pitchtools.PitchSegment)



# -*- coding: utf-8 -*-
import re
import six


hyphen_delimited_lowercase_regex_body = '(([a-z,0-9]+[-]+)*[a-z,0-9]+)?'
hyphen_delimited_lowercase_regex = re.compile(
    '^{}$'.format(hyphen_delimited_lowercase_regex_body),
    re.VERBOSE,
    )


def is_dash_case(argument):
    r'''Is true when `argument` is a string and is hyphen delimited lowercase.

    ..  container:: example

        ::

            >>> stringtools.is_dash_case('foo-bar')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_dash_case('foo bar')
            False

    Returns true or false.
    '''
    if not isinstance(argument, six.string_types):
        return False
    return bool(hyphen_delimited_lowercase_regex.match(argument))



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.selectiontools import Selection


def test_selectiontools_Selection__all_are_contiguous_components_in_same_parent_01():
    r'''Is true for strictly contiguous leaves in voice.
    Is false for other time orderings of leaves in voice.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")

    assert Selection._all_are_contiguous_components_in_same_parent(voice[:])

    assert not Selection._all_are_contiguous_components_in_same_parent(
        list(reversed(voice[:])))

    components = []
    components.extend(voice[2:])
    components.extend(voice[:2])
    assert not Selection._all_are_contiguous_components_in_same_parent(
        components)

    components = []
    components.extend(voice[3:4])
    components.extend(voice[:1])
    assert not Selection._all_are_contiguous_components_in_same_parent(
        components)
    components = [voice]
    components.extend(voice[:])
    assert not Selection._all_are_contiguous_components_in_same_parent(
        components)


def test_selectiontools_Selection__all_are_contiguous_components_in_same_parent_02():
    r'''Is true for unincorporated components when orphans allowed.
    Is false for unincorporated components when orphans not allowed.
    '''

    voice = Voice(r'''
        {
            c'8
            d'8
        }
        {
            e'8
            f'8
        }
        ''')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
        }
        '''
        )

    assert Selection._all_are_contiguous_components_in_same_parent([voice])
    assert not Selection._all_are_contiguous_components_in_same_parent(
        [voice], allow_orphans = False)

    assert Selection._all_are_contiguous_components_in_same_parent(voice[:])

    assert Selection._all_are_contiguous_components_in_same_parent(voice[0][:])
    assert Selection._all_are_contiguous_components_in_same_parent(voice[1][:])

    leaves = select(voice).by_leaf()
    assert not Selection._all_are_contiguous_components_in_same_parent(leaves)


def test_selectiontools_Selection__all_are_contiguous_components_in_same_parent_03():
    r'''Is true for orphan leaves when allow_orphans is true.
    Is false for orphan leaves when allow_orphans is false.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]

    assert Selection._all_are_contiguous_components_in_same_parent(notes)
    assert not Selection._all_are_contiguous_components_in_same_parent(
        notes, allow_orphans=False)


def test_selectiontools_Selection__all_are_contiguous_components_in_same_parent_04():
    r'''Empty list returns true.
    '''

    sequence = []

    assert Selection._all_are_contiguous_components_in_same_parent(sequence)



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools.topleveltools import attach


def apply_page_breaks(score):
    r'''Applies page breaks to score.
    '''

    bell_voice = score['Bell Voice']

    measure_indices = [
        5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 72,
        79, 86, 93, 100,
        ]

    for measure_index in measure_indices:
        command = indicatortools.LilyPondCommand('break', 'after')
        attach(command, bell_voice[measure_index])


# -*- coding: utf-8 -*-
import bisect
import copy
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import select


class QGrid(AbjadObject):
    r'''A rhythm-tree-based model for how millisecond attack points
    collapse onto the offsets generated by a nested rhythmic structure:

    ::

        >>> q_grid = quantizationtools.QGrid()

    ::

        >>> print(format(q_grid, 'storage'))
        quantizationtools.QGrid(
            root_node=quantizationtools.QGridLeaf(
                preprolated_duration=durationtools.Duration(1, 1),
                is_divisible=True,
                ),
            next_downbeat=quantizationtools.QGridLeaf(
                preprolated_duration=durationtools.Duration(1, 1),
                is_divisible=True,
                ),
            )

    ``QGrids`` model not only the internal nodes of the nesting
    structure, but also the downbeat to the "next" ``QGrid``,
    allowing events which occur very late within one structure
    to collapse virtually onto the beginning of the next structure.

    ``QEventProxies`` can be "loaded in" to the node contained
    by the ``QGrid`` closest to their virtual offset:

    ::

        >>> q_event_a = quantizationtools.PitchedQEvent(250, [0])
        >>> q_event_b = quantizationtools.PitchedQEvent(750, [1])
        >>> proxy_a = quantizationtools.QEventProxy(q_event_a, 0.25)
        >>> proxy_b = quantizationtools.QEventProxy(q_event_b, 0.75)

    ::

        >>> q_grid.fit_q_events([proxy_a, proxy_b])

    ::

        >>> for q_event_proxy in q_grid.root_node.q_event_proxies:
        ...     print(format(q_event_proxy, 'storage'))
        ...
        quantizationtools.QEventProxy(
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(250, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                ),
            durationtools.Offset(1, 4)
            )

    ::

        >>> for q_event_proxy in q_grid.next_downbeat.q_event_proxies:
        ...     print(format(q_event_proxy, 'storage'))
        ...
        quantizationtools.QEventProxy(
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(750, 1),
                pitches=(
                    pitchtools.NamedPitch("cs'"),
                    ),
                ),
            durationtools.Offset(3, 4)
            )

    Used internally by the ``Quantizer``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_next_downbeat',
        '_root_node',
        )

    ### INITIALIZATION ###

    def __init__(self, root_node=None, next_downbeat=None):
        from abjad.tools import quantizationtools
        if root_node is None:
            root_node = quantizationtools.QGridLeaf(preprolated_duration=1)
        assert isinstance(root_node,
            (quantizationtools.QGridLeaf, quantizationtools.QGridContainer))
        if next_downbeat is None:
            next_downbeat = quantizationtools.QGridLeaf(
                preprolated_duration=1)
        assert isinstance(next_downbeat, quantizationtools.QGridLeaf)
        self._root_node = root_node
        self._next_downbeat = next_downbeat
        self._next_downbeat._offset = durationtools.Offset(1)
        self._next_downbeat._offsets_are_current = True

    ### SPECIAL METHODS ###

    def __call__(self, beatspan):
        r'''Calls q-grid.
        '''
        result = self.root_node(beatspan)
        result_leaves = []
        for x in result:
            if isinstance(x, scoretools.Container):
                leaves = select(x).by_leaf()
                result_leaves.extend(leaves)
            else:
                result_leaves.append(x)
        for result_leaf, q_grid_leaf in zip(result_leaves, self.leaves[:-1]):
            if q_grid_leaf.q_event_proxies:
                q_events = [q_event_proxy.q_event
                    for q_event_proxy in q_grid_leaf.q_event_proxies]
                q_events.sort(
                    key=lambda x: 0 if x.index is None else x.index)
                annotation = indicatortools.Annotation(
                    'q_events', tuple(q_events))
                attach(annotation, result_leaf)
        return result

    def __copy__(self, *arguments):
        r'''Copies q-grid.

        Returns new q-grid.
        '''
        root_node, next_downbeat = self._root_node, self._next_downbeat
        return type(self)(copy.copy(root_node), copy.copy(next_downbeat))

    def __eq__(self, argument):
        r'''True if `argument` is a q-grid with root node and next downbeat equal
        to those of this q-grid. Otherwise false.

        Returns true or false.
        '''
        if type(self) == type(argument):
            if self.root_node == argument.root_node:
                if self.next_downbeat == argument.next_downbeat:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats q-event.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __hash__(self):
        r'''Hashes q-grid.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(QGrid, self).__hash__()

    ### PUBLIC METHODS ###

    def fit_q_events(self, q_event_proxies):
        r'''Fit each ``QEventProxy`` in ``q_event_proxies`` onto
        the contained ``QGridLeaf`` whose offset is nearest.

        Returns None
        '''
        from abjad.tools import quantizationtools
        assert all(isinstance(x, quantizationtools.QEventProxy)
            for x in q_event_proxies)
        leaves, offsets = self.leaves, self.offsets
        for q_event_proxy in q_event_proxies:
            idx = bisect.bisect_left(offsets, q_event_proxy.offset)
            if q_event_proxy.offset == offsets[idx]:
                leaves[idx].q_event_proxies.append(q_event_proxy)
            else:
                left, right = offsets[idx - 1], offsets[idx]
                left_diff = abs(left - q_event_proxy.offset)
                right_diff = abs(right - q_event_proxy.offset)
                if right_diff < left_diff:
                    leaves[idx].q_event_proxies.append(q_event_proxy)
                else:
                    leaves[idx - 1].q_event_proxies.append(q_event_proxy)

    def sort_q_events_by_index(self):
        r'''Sort ``QEventProxies`` attached to each ``QGridLeaf`` in a
        ``QGrid`` by their index.

        Returns None.
        '''
        for leaf in self.leaves:
            leaf.q_event_proxies.sort(key=lambda x: x.index)

    def subdivide_leaf(self, leaf, subdivisions):
        r'''Replace the ``QGridLeaf`` ``leaf`` contained in a ``QGrid``
        by a ``QGridContainer`` containing ``QGridLeaves`` with durations
        equal to the ratio described in ``subdivisions``

        Returns the ``QEventProxies`` attached to ``leaf``.
        '''
        from abjad.tools import quantizationtools
        container = quantizationtools.QGridContainer(
            preprolated_duration=leaf.preprolated_duration,
            children=[
                quantizationtools.QGridLeaf(preprolated_duration=subdivision)
                for subdivision in subdivisions
                ])
        if leaf.parent is not None:
            index = leaf.parent.index(leaf)
            leaf.parent[index] = container
        # otherwise, our root node if just a QGridLeaf
        else:
            self._root_node = container
        return leaf.q_event_proxies

    def subdivide_leaves(self, pairs):
        r'''Given a sequence of leaf-index:subdivision-ratio pairs ``pairs``,
        subdivide the ``QGridLeaves`` described by the indices into
        ``QGridContainers`` containing ``QGridLeaves`` with durations
        equal to their respective subdivision-ratios.

        Returns the ``QEventProxies`` attached to thus subdivided
        ``QGridLeaf``.
        '''
        pairs = sorted(dict(pairs).items())
        leaf_indices = [pair[0] for pair in pairs]
        subdivisions = [pair[1] for pair in pairs]

        all_leaves = self.leaves
        leaves_to_subdivide = [all_leaves[idx] for idx in leaf_indices]

        q_event_proxies = []
        for i, leaf in enumerate(leaves_to_subdivide):

            next_leaf = all_leaves[all_leaves.index(leaf) + 1]
            if next_leaf is self.next_downbeat:
                next_leaf_offset = durationtools.Offset(1)
            else:
                next_leaf_offset = next_leaf.start_offset

            q_event_proxies.extend(self.subdivide_leaf(leaf, subdivisions[i]))
            for q_event_proxy in tuple(next_leaf.q_event_proxies):
                if q_event_proxy.offset < next_leaf_offset:
                    idx = next_leaf.q_event_proxies.index(q_event_proxy)
                    q_event_proxies.append(next_leaf.q_event_proxies.pop(idx))

        return q_event_proxies

    ### PUBLIC PROPERTIES ###

    @property
    def distance(self):
        r'''The computed total distance of the offset of each ``QEventProxy``
        contained by the ``QGrid`` to the offset of the ``QGridLeaf`` to
        which the ``QEventProxy`` is attached.

        Return ``Duration`` instance.
        '''
        count = 0
        absolute_distance = 0
        for leaf, offset in zip(self.leaves, self.offsets):
            for q_event_proxy in leaf.q_event_proxies:
                absolute_distance += abs(q_event_proxy.offset - offset)
                count += 1
        if count:
            return absolute_distance / count
        return None

    @property
    def leaves(self):
        r'''All of the leaf nodes in the QGrid, including the next
        downbeat's node.

        Returns tuple of ``QGridLeaf`` instances.
        '''
        from abjad.tools import quantizationtools
        if isinstance(self._root_node, quantizationtools.QGridLeaf):
            return (self._root_node, self._next_downbeat)
        return self._root_node.leaves + (self._next_downbeat,)

    @property
    def next_downbeat(self):
        r'''The node representing the "next" downbeat after the contents
        of the QGrid's tree.

        Return ``QGridLeaf`` instance.
        '''
        return self._next_downbeat

    @property
    def offsets(self):
        r'''The offsets between 0 and 1 of all of the leaf nodes in the QGrid.

        Returns tuple of ``Offset`` instances.
        '''
        return tuple([x.start_offset
            for x in self.leaves[:-1]] + [durationtools.Offset(1)])

    @property
    def pretty_rtm_format(self):
        r'''The pretty RTM-format of the root node of the ``QGrid``.

        Returns string.
        '''
        return self._root_node.pretty_rtm_format

    @property
    def root_node(self):
        r'''The root node of the ``QGrid``.

        Return ``QGridLeaf`` or ``QGridContainer``.
        '''
        return self._root_node

    @property
    def rtm_format(self):
        r'''The RTM format of the root node of the ``QGrid``.

        Returns string.
        '''
        return self._root_node.rtm_format



# -*- coding: utf-8 -*-


def all_are_nonnegative_integer_equivalent_numbers(argument):
    '''Is true when `argument` is an iterable collection of nonnegative
    integer-equivalent numbers. Otherwise false.

    ..  container:: example

        ::

            >>> items = [0, 0.0, Fraction(0), 2, 2.0, Fraction(2)]
            >>> mathtools.all_are_nonnegative_integer_equivalent_numbers(items)
            True

        ::

            >>> items = [0, 0.0, Fraction(0), -2, 2.0, Fraction(2)]
            >>> mathtools.all_are_nonnegative_integer_equivalent_numbers(items)
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    try:
        return all(
            mathtools.is_nonnegative_integer_equivalent_number(_)
            for _ in argument
            )
    except TypeError:
        return False



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSegment_has_duplicates_01():
    r'''Works with chords.
    '''

    assert pitchtools.PitchSegment.from_selection(
        Chord([13, 13, 14], (1, 4))).has_duplicates()
    assert not pitchtools.PitchSegment.from_selection(
        Chord([13, 14], (1, 4))).has_duplicates()
    assert not pitchtools.PitchSegment.from_selection(
        Chord([], (1, 4))).has_duplicates()


def test_pitchtools_PitchSegment_has_duplicates_02():
    r'''Works with notes, rests and skips.
    '''

    assert not pitchtools.PitchSegment.from_selection(
        Note(13, (1, 4))).has_duplicates()
    assert not pitchtools.PitchSegment.from_selection(
        Rest((1, 4))).has_duplicates()
    assert not pitchtools.PitchSegment.from_selection(
        scoretools.Skip((1, 4))).has_duplicates()


def test_pitchtools_PitchSegment_has_duplicates_03():
    r'''Works with containers.
    '''

    staff = Staff("c'8 c'8 c'8 c'8")
    assert pitchtools.PitchSegment.from_selection(staff).has_duplicates()

    staff = Staff("c'8 d'8 e'8 f'8")
    assert not pitchtools.PitchSegment.from_selection(staff).has_duplicates()



# -*- coding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import schemetools
from abjad.tools.abctools import AbjadValueObject


class ExampleWrapper(AbjadValueObject):
    r'''Example wrapper.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Illustration helpers'

    __slots__ = (
        '_division_lists',
        '_arguments',
        )

    ### INITIALIZER ###

    def __init__(self, arguments=None, division_lists=None):
        division_lists = division_lists or ()
        self._arguments = arguments
        self._division_lists = tuple(division_lists)

    ### PRIVATE METHODS ###

    def _to_markup(self, class_):
        instance = class_(**self.arguments)
        string = format(instance, 'storage')
        string = string.replace('rhythmmakertools.', '')
        lines = string.split('\n')
        command = markuptools.MarkupCommand('column', lines)
        command.force_quotes = True
        pair = schemetools.SchemePair('font-name', 'Courier')
        command = markuptools.MarkupCommand('override', pair, command)
        markup = markuptools.Markup(command, direction=Up)
        return markup

    ### PUBLIC PROPERTIES ###

    @property
    def arguments(self):
        r'''Gets arguments.

        Returns dictionary.
        '''
        return self._arguments

    @property
    def division_lists(self):
        r'''Gets division lists.

        Returns tuple.
        '''
        return self._division_lists


# -*- coding: utf-8 -*-
import abjad
from abjad.tools import scoretools


def make_desordre_staff(pitches):
    r'''Makes Désordre staff.
    '''

    staff = scoretools.Staff()
    for sequence in pitches:
        measure = abjad.demos.desordre.make_desordre_measure(sequence)
        staff.append(measure)
    return staff


# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools.topleveltools import attach


def apply_rehearsal_marks(score):
    r'''Applies rehearsal marks to score.
    '''

    bell_voice = score['Bell Voice']

    measure_indices = [
        6, 12, 18, 24, 30, 36, 42, 48, 54, 60, 66, 72, 78, 84,
        90, 96, 102,
        ]

    for measure_index in measure_indices:
        command = indicatortools.LilyPondCommand(r'mark \default', 'before')
        attach(command, bell_voice[measure_index])


# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate


def move_measure_prolation_to_full_measure_tuplet(argument):
    '''Move measure prolation to full-measure tuplet.

    Turn non-power-of-two measures into power-of-two measures containing
    a single fixed-duration tuplet.

    Note that not all non-power-of-two measures can be made power-of-two.

    Returns None because processes potentially many measures.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools

    for measure in iterate(argument).by_class(scoretools.Measure):
        effective_time_signature = measure.time_signature
        if effective_time_signature.has_non_power_of_two_denominator:

            # find time signature and contents multipliers
            time_signature_multiplier = effective_time_signature.implied_prolation
            contents_multiplier = \
                measure._get_likely_multiplier_of_components(measure[:])

            # update non-power-of-two time signature to power-of-two
            power_of_two_time_signature = effective_time_signature.with_power_of_two_denominator(
                contents_multiplier)
            detach(indicatortools.TimeSignature, measure)
            attach(power_of_two_time_signature, measure)

            # find target duration and create tuplet
            target_duration = time_signature_multiplier * measure._contents_duration
            tuplet = scoretools.FixedDurationTuplet(target_duration, measure[:])

            # scale tuplet contents, if helpful
            if contents_multiplier is not None:
                numerator = contents_multiplier.numerator
                denominator = contents_multiplier.denominator
                pair = (denominator, numerator)
                inverse_multiplier = durationtools.Multiplier(*pair)
                tuplet._scale_contents(inverse_multiplier)



# -*- coding: utf-8 -*-
from abjad.tools import documentationtools
from abjad.demos.desordre.make_desordre_pitches import make_desordre_pitches
from abjad.demos.desordre.make_desordre_score import make_desordre_score


def make_desordre_lilypond_file():
    '''Makes Désordre LilyPond file.
    '''

    pitches = make_desordre_pitches()
    score = make_desordre_score(pitches)
    lilypond_file = documentationtools.make_ligeti_example_lilypond_file(score)
    return lilypond_file



# -*- coding: utf-8 -*-
from abjad.tools import abctools


class CodeBlockSpecifier(abctools.AbjadValueObject):
    r'''A code block specifier.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Internals'

    __slots__ = (
        '_allow_exceptions',
        '_hide',
        '_strip_prompt',
        '_text_width',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        allow_exceptions=None,
        hide=None,
        strip_prompt=None,
        text_width=None,
        ):
        self._allow_exceptions = bool(allow_exceptions) or None
        self._hide = bool(hide) or None
        self._strip_prompt = bool(strip_prompt) or None
        if text_width is not None:
            if text_width is True:
                text_width = 80
            try:
                text_width = int(text_width)
                if text_width < 1:
                    text_width = None
            except:
                text_width = None
        self._text_width = text_width

    ### PUBLIC METHODS ###

    @classmethod
    def from_options(cls, **options):
        r'''Creates code block specifier from `options` dictionary.

        Returns code block specifier.
        '''
        allow_exceptions = options.get('allow_exceptions', None) or None
        hide = options.get('hide', None) or None
        strip_prompt = options.get('strip_prompt', None) or None
        text_width = options.get('text_width', None) or None
        if all(_ is None for _ in (
            allow_exceptions,
            hide,
            strip_prompt,
            text_width,
            )):
            return None
        return cls(
            allow_exceptions=allow_exceptions,
            hide=hide,
            strip_prompt=strip_prompt,
            text_width=text_width,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def allow_exceptions(self):
        r'''Is true if code block allows exceptions. Otherwise false.

        Returns true or false.
        '''
        return self._allow_exceptions

    @property
    def hide(self):
        r'''Is true if code block should be hidden. Otherwise false.

        Returns true or false.
        '''
        return self._hide

    @property
    def strip_prompt(self):
        r'''Is true if code block should strip Python prompt from output.
        Otherwise false.

        Returns true or false.
        '''
        return self._strip_prompt

    @property
    def text_width(self):
        r'''Gets text width wrap of code block.

        Returns integer or none.
        '''
        return self._text_width



# -*- coding: utf-8 -*-
import six
import textwrap


def normalize(string, indent=None):
    r"""Normalizes `string`.

    ::

        >>> string = r'''
        ...     foo
        ...         bar
        ... '''
        >>> print(string)
        <BLANKLINE>
            foo
                bar
        <BLANKLINE>

    ::

        >>> print(stringtools.normalize(string))
        foo
            bar

    ::

        >>> print(stringtools.normalize(string, indent=4))
            foo
                bar

    ::

        >>> print(stringtools.normalize(string, indent='* '))
        * foo
        *     bar

    Returns string.
    """
    string = string.replace('\t', '    ')
    lines = string.split('\n')
    while lines and (not lines[0] or lines[0].isspace()):
        lines.pop(0)
    while lines and (not lines[-1] or lines[-1].isspace()):
        lines.pop()
    for i, line in enumerate(lines):
        lines[i] = line.rstrip()
    string = '\n'.join(lines)
    string = textwrap.dedent(string)
    if indent:
        if not isinstance(indent, six.string_types):
            indent = ' ' * abs(int(indent))
        lines = string.split('\n')
        for i, line in enumerate(lines):
            if line:
                lines[i] = '{}{}'.format(indent, line)
        string = '\n'.join(lines)
    return string



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment_multiply_01():

    pitch_class_segment = pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])

    assert pitch_class_segment.multiply(0) == \
        pitchtools.PitchClassSegment([0, 0, 0, 0, 0, 0])
    assert pitch_class_segment.multiply(1) == \
        pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])
    assert pitch_class_segment.multiply(5) == \
        pitchtools.PitchClassSegment([0, 6, 2, 8, 9, 10])
    assert pitch_class_segment.multiply(7) == \
        pitchtools.PitchClassSegment([0, 6, 10, 4, 3, 2])



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_RomanNumeral_symbolic_string_01():

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'dominant', 7, 0, (4, 3))
    assert roman_numeral.symbolic_string == 'V7/4-3'

    roman_numeral = tonalanalysistools.RomanNumeral(2, 'minor', 7, 1)
    assert roman_numeral.symbolic_string == 'ii6/5'

    roman_numeral = tonalanalysistools.RomanNumeral(1, 'major', 5, 1)
    assert roman_numeral.symbolic_string == 'I6'



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_RomanNumeral___repr___01():

    harmony = tonalanalysistools.RomanNumeral(1, 'major', 5, 0)
    assert repr(harmony) == 'IMajorTriadInRootPosition'

    harmony = tonalanalysistools.RomanNumeral(5, 'dominant', 7, 2)
    assert repr(harmony) == 'VDominantSeventhInSecondInversion'

    harmony = tonalanalysistools.RomanNumeral(('flat', 2), 'major', 5, 1)
    assert repr(harmony) == 'FlatIIMajorTriadInFirstInversion'

    harmony = tonalanalysistools.RomanNumeral(1, 'minor', 5, 0)
    assert repr(harmony) == 'iMinorTriadInRootPosition'



# -*- coding: utf-8 -*-
import re
import six
from abjad.tools.stringtools.is_snake_case \
    import underscore_delimited_lowercase_regex_body


underscore_delimited_lowercase_file_name_with_extension_regex_body = """
    {}
    \.
    [a-z,0-9]+
    """.format(underscore_delimited_lowercase_regex_body)

underscore_delimited_lowercase_file_name_with_extension_regex = re.compile(
    '^{}$'.format(underscore_delimited_lowercase_file_name_with_extension_regex_body),
    re.VERBOSE,
    )


def is_snake_case_file_name_with_extension(argument):
    r'''Is true when `argument` is a string and is underscore-delimited lowercase
    file name with extension.

    ..  container:: example

        ::

            >>> stringtools.is_snake_case_file_name_with_extension('foo_bar.blah')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_snake_case_file_name_with_extension('foo.bar.blah')
            False

    Returns true or false.
    '''
    if not isinstance(argument, six.string_types):
        return False
    if argument == '':
        return True
    return bool(underscore_delimited_lowercase_file_name_with_extension_regex.match(argument))



# -*- coding: utf-8 -*-


def run_ajv():
    r'''Entry point for setuptools.

    One-line wrapper around AbjDevScript.
    '''
    from abjad.tools import commandlinetools

    commandlinetools.AbjDevScript()()



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class SkipRhythmMaker(RhythmMaker):
    r'''Skip rhythm-maker.

    ..  container:: example

        Makes skips equal to the duration of input divisions.

        ::

            >>> rhythm_maker = rhythmmakertools.SkipRhythmMaker()

        ::

            >>> divisions = [(1, 4), (3, 16), (5, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 1/4
                    s1 * 1/4
                }
                {
                    \time 3/16
                    s1 * 3/16
                }
                {
                    \time 5/8
                    s1 * 5/8
                }
            }

    Usage follows the two-step configure-once / call-repeatedly pattern shown
    here.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Rhythm-makers'

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __call__(self, divisions, rotation=None):
        r'''Calls skip rhythm-maker on `divisions`.

        Returns list of selections.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            rotation=rotation,
            )

    def __format__(self, format_specification=''):
        r'''Formats skip rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ..  container:: example

            ::

                >>> f(rhythm_maker)
                rhythmmakertools.SkipRhythmMaker()

        Returns string.
        '''
        superclass = super(SkipRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _make_music(self, divisions, rotation):
        result = []
        for division in divisions:
            prototype = mathtools.NonreducedFraction
            assert isinstance(division, prototype), repr(division)
            written_duration = durationtools.Duration(1)
            multiplied_duration = division
            skip = scoretools.make_skips(
                written_duration, [multiplied_duration])
            result.append(skip)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def tuplet_spelling_specifier(self):
        r'''Gets tuplet spelling specifier.

        ..  note:: not yet implemented.

        Returns tuplet spelling specifier or none.
        '''
        superclass = super(SkipRhythmMaker, self)
        return superclass.tuplet_spelling_specifier



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSet___eq___01():
    r'''Named pitch-class set equality works as expected.
    '''

    pitch_class_set_1 = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),])

    pitch_class_set_2 = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),])

    pitch_class_set_3 = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('e'),
        pitchtools.NamedPitchClass('f'),
        pitchtools.NamedPitchClass('g'),])

    assert pitch_class_set_1 == pitch_class_set_1
    assert pitch_class_set_1 == pitch_class_set_2
    assert pitch_class_set_1 != pitch_class_set_3
    assert pitch_class_set_2 != pitch_class_set_3


def test_pitchtools_PitchClassSet___eq___02():
    r'''PCset equality works as expected.
    '''

    pitch_class_set_1 = pitchtools.PitchClassSet([0, 2, 6, 7])
    pitch_class_set_2 = pitchtools.PitchClassSet([0, 2, 6, 7])
    pitch_class_set_3 = pitchtools.PitchClassSet([0, 2, 6, 8])

    assert pitch_class_set_1 == pitch_class_set_2
    assert pitch_class_set_1 != pitch_class_set_3
    assert pitch_class_set_2 != pitch_class_set_3
    assert not pitch_class_set_1 != pitch_class_set_2



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitchClass_apply_accidental_01():

    pc = pitchtools.NamedPitchClass('cs')

    assert pc.apply_accidental('sharp') == pitchtools.NamedPitchClass('css')
    assert pc.apply_accidental('flat') == pitchtools.NamedPitchClass('c')
    assert pc.apply_accidental('natural') == pitchtools.NamedPitchClass('cs')



# -*- coding: utf-8 -*-
from abjad import abjad_configuration
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    @mock.patch('abjad.commandlinetools.ScorePackageScript._call_subprocess')
    def test_success(self, call_subprocess_mock):
        call_subprocess_mock.return_value = 0
        self.create_score()
        material_path = self.create_material('test_material')
        script = commandlinetools.ManageMaterialScript()
        command = ['--edit', 'test_material']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
        Edit candidates: 'test_material' ...
        ''')
        definition_path = material_path.joinpath('definition.py')
        command = '{} {!s}'.format(
            abjad_configuration.get_text_editor(),
            definition_path,
            )
        call_subprocess_mock.assert_called_with(command)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__containers__Container_01():
    parser = LilyPondParser()
    target = Container()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitch___float___01():
    r'''Returns pitch number of 12-ET numbered pitch as float.
    '''

    numbered_pitch = pitchtools.NumberedPitch(13)
    assert isinstance(float(numbered_pitch), float)
    assert float(numbered_pitch) == 13.0


def test_pitchtools_NumberedPitch___float___02():
    r'''Returns pitch number of 24-ET numbered pitch as float.
    '''

    numbered_pitch = pitchtools.NumberedPitch(13.5)
    assert isinstance(float(numbered_pitch), float)
    assert float(numbered_pitch) == 13.5



# -*- coding: utf-8 -*-
import shutil
import tempfile
from abjad.tools.abctools.ContextManager import ContextManager


class TemporaryDirectory(ContextManager):
    r'''A temporary directory context manager.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Context managers'

    __slots__ = (
        '_parent_directory_path',
        '_temporary_directory_path',
        )

    ### INITIALIZER ###

    def __init__(self, parent_directory_path=None):
        self._parent_directory_path = parent_directory_path
        self._temporary_directory_path = None

    ### SPECIAL METHODS ###

    def __enter__(self):
        r'''Enters context manager.

        Creates and returns path to a temporary directory.
        '''
        self._temporary_directory_path = tempfile.mkdtemp(
            dir=self.parent_directory_path,
            )
        return self._temporary_directory_path

    def __exit__(self, exc_type, exc_value, traceback):
        r'''Exits context manager.

        Deletes previously created temporary directory.
        '''
        shutil.rmtree(self._temporary_directory_path)

    ### PUBLIC PROPERTIES ###

    @property
    def parent_directory_path(self):
        r'''Gets parent directory path.

        Returns string.
        '''
        return self._parent_directory_path

    @property
    def temporary_directory_path(self):
        r'''Gets temporary directory path.

        Returns string.
        '''
        return self._temporary_directory_path



# -*- coding: utf-8 -*-
from abjad.tools import rhythmtreetools
import copy


def test_rhythmtreetools_RhythmTreeContainer___copy___01():

    string = '(1 (1 (2 (3 (4 (1 1 1)))) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(string)[0]
    copied = copy.copy(tree)

    assert format(tree) == format(copied)
    assert tree is not copied

    assert format(tree[0]) == format(copied[0])
    assert tree[0] is not copied[0]

    assert format(tree[1]) == format(copied[1])
    assert tree[1] is not copied[1]

    assert format(tree[2]) == format(copied[2])
    assert tree[2] is not copied[2]



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Multiplication(AbjadValueObject):
    r'''Multiplication operator.

    ..  container:: example

        ::

            >>> Multiplication()
            Multiplication(n=1)

    ..  container:: example

        ::

            >>> Multiplication(n=5)
            Multiplication(n=5)

    Object model of twelve-tone multiplication operator.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        '_n',
        )

    ### INITIALIZER ###

    def __init__(self, n=1):
        self._n = n

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r'''Composes multiplication and `operator`.

        ..  container:: example

            Example segment:

            ::

                >>> items = [0, 2, 4, 5]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP
    
            Example operators:

            ::

                >>> multiplication = Multiplication(n=5)
                >>> transposition = Transposition(n=3)

        ..  container:: example


            Transposition followed by multiplication:

            ::

                >>> operator = multiplication + transposition
                >>> str(operator)
                'M5T3'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    ef'8
                    cs'8
                    b'8
                    e'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Same as above because multiplication and transposition commute:

            ::

                >>> operator = transposition + multiplication
                >>> str(operator)
                'T3M5'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    ef'8
                    cs'8
                    b'8
                    e'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        Returns compound operator.
        '''
        from abjad.tools import pitchtools
        return pitchtools.CompoundOperator._compose_operators(self, operator)

    def __call__(self, argument):
        r'''Calls multiplication on `argument`.

        ..  container:: example

            Multiplies pitch-class:

            ::

                >>> multiplication = Multiplication(n=5)
                >>> pitch_class = NumberedPitchClass(4)
                >>> multiplication(pitch_class)
                NumberedPitchClass(8)

        ..  container:: example

            Multiplies pitch:

            ::

                >>> multiplication = Multiplication(n=7)
                >>> pitch = NamedPitch("f'")
                >>> multiplication(pitch)
                NamedPitch("b'")

        Returns new object with type equal to that of `argument`.
        '''
        if hasattr(argument, 'multiply'):
            result = argument.multiply(self.n)
        else:
            message = 'do not know how to multiply: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        return result

    def __str__(self):
        r'''Gets string representation of operator.

        ..  container:: example

            ::

                >>> str(Multiplication())
                'M1'

        ..  container:: example

            ::

                >>> str(Multiplication(n=5))
                'M5'

        '''
        string = 'M{}'
        string = string.format(self.n)
        return string

    ### PRIVATE METHODS ###

    def _get_markup(self, direction=None):
        from abjad.tools import markuptools
        operator = markuptools.Markup('M', direction=direction)
        subscript = markuptools.Markup(self.n).sub()
        markup = markuptools.Markup.concat([operator, subscript])
        return markup

    def _is_identity_operator(self):
        if self.n == 1:
            return True
        return False 

    ### PUBLIC PROPERTIES ###

    @property
    def n(self):
        r'''Gets index of multiplication.

        ..  container:: example

            ::

                >>> multiplication = Multiplication()
                >>> multiplication.n
                1

        ..  container:: example

            ::

                >>> multiplication = Multiplication(n=5)
                >>> multiplication.n
                5

        Set to integer or none.
        '''
        return self._n



# -*- coding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadValueObject


class FlattenSelectorCallback(AbjadValueObject):
    r'''Flatten selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_depth',
        )

    ### INITIALIZER ###

    def __init__(self, depth=-1):
        self._depth = depth

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.
        '''
        assert isinstance(argument, tuple), repr(argument)
        argument = sequencetools.Sequence(argument)
        argument = argument.flatten(depth=self.depth)
        return tuple(argument)

    ### PUBLIC PROPERTIES ###

    @property
    def depth(self):
        r'''Gets depth of callback.

        Returns integer.
        '''
        return self._depth



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import spannertools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import select


def edit_second_violin_voice(score, durated_reservoir):
    r'''Edits second violin voice.
    '''

    voice = score['Second Violin Voice']
    descents = durated_reservoir['Second Violin']

    last_descent = select(descents[-1])
    copied_descent = mutate(last_descent).copy()
    copied_descent = list(copied_descent)
    copied_descent[-1].written_duration = durationtools.Duration(1, 1)
    copied_descent.append(scoretools.Note('a2'))
    for leaf in copied_descent:
        articulation = indicatortools.Articulation('accent')
        attach(articulation, leaf)
        articulation = indicatortools.Articulation('tenuto')
        attach(articulation, leaf)
    voice.extend(copied_descent)

    final_sustain = []
    for _ in range(32):
        final_sustain.append(scoretools.Note('a1.'))
    final_sustain.append(scoretools.Note('a2'))
    articulation = indicatortools.Articulation('accent')
    attach(articulation, final_sustain[0])
    articulation = indicatortools.Articulation('tenuto')
    attach(articulation, final_sustain[0])

    voice.extend(final_sustain)
    tie = spannertools.Tie()
    attach(tie, final_sustain)
    voice.extend('r4 r2.')


# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_MeasuredComplexBeam_01():

    staff = Staff("abj: | 2/16 c'16 d'16 || 2/16 e'16 f'16 |"
        "| 2/16 g'16 a'16 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/16
                c'16
                d'16
            }
            {
                e'16
                f'16
            }
            {
                g'16
                a'16
            }
        }
        '''
        )

    leaves = select(staff).by_leaf()
    beam = spannertools.MeasuredComplexBeam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/16
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #2
                c'16 [
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #1
                d'16
            }
            {
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #2
                e'16
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #1
                f'16
            }
            {
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #2
                g'16
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #0
                a'16 ]
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner___contains___01():

    class MockSpanner(spannertools.Spanner):

        def __init__(self, components=None):
            spannertools.Spanner.__init__(self, components)

    note = Note("c'4")

    spanner = MockSpanner()
    attach(spanner, Note("c'4"))

    assert not note in spanner



# -*- coding: utf-8 -*-
import os
import subprocess
import sys
import traceback
from abjad.tools.systemtools.Configuration import Configuration


class AbjadConfiguration(Configuration):
    r'''Abjad configuration.

    ..  container:: example

        ::

            >>> abjad_configuration = systemtools.AbjadConfiguration()

        ::

            >>> abjad_configuration['accidental_spelling']
            'mixed'

    ..  container:: example

        Behavior at instantiation:

        * Looks for ``$HOME/.abjad/``.

        * Creates ``$HOME/.abjad/`` if directory does not exist.
        
        * Looks for ``$HOME/.abjad/abjad.cfg``.
        
        * Creates ``$HOME/.abjad/abjad.cfg`` if file does not exist.

        * Parses ``$HOME/.abjad/abjad.cfg``.

        * Provides default key-value pairs for pairs which fail validation.

        * Writes configuration changes to disk.

        * Creates Abjad output directory if directory does not exist.

    Supports mutable mapping dictionary interface.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'System configuration'

    # for caching
    _lilypond_version_string = None

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self):
        Configuration.__init__(self)
        if not os.path.exists(self.abjad_output_directory):
            try:
                os.makedirs(self.abjad_output_directory)
            except (IOError, OSError):
                traceback.print_exc()

    ### PRIVATE METHODS ###

    def _get_option_definitions(self):
        options = {
            'abjad_output_directory': {
                'comment': [
                    'Set to the directory where all Abjad-generated files',
                    '(such as PDFs and LilyPond files) should be saved.',
                    'Defaults to $HOME/.abjad/output/'
                    ],
                'default': os.path.join(
                    self.configuration_directory_path,
                    'output',
                    ),
                'validator': str,
                },
            'accidental_spelling': {
                'comment': [
                    'Default accidental spelling (mixed|sharps|flats).',
                    ],
                'default': 'mixed',
                'validator': lambda x: x in ('mixed', 'sharps', 'flats'),
                },
            'lilypond_path': {
                'comment': [
                    'Lilypond executable path. Set to override dynamic lookup.'
                    ],
                'default': 'lilypond',
                'validator': str,
                },
            'midi_player': {
                'comment': [
                    'MIDI player to open MIDI files.',
                    'When unset your OS should know how to open MIDI files.',
                    ],
                'default': None,
                'validator': str,
                },
            'pdf_viewer': {
                'comment': [
                    'PDF viewer to open PDF files.',
                    'When unset your OS should know how to open PDFs.',
                    ],
                'default': None,
                'validator': str,
                },
            'text_editor': {
                'comment': [
                    'Text editor to edit text files.',
                    'When unset your OS should know how to open text files.'
                    ],
                'default': None,
                'validator': str,
                },
            }
        return options

    ### PUBLIC METHODS ###

    @classmethod
    def get_abjad_startup_string(class_):
        r'''Gets Abjad startup string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_abjad_startup_string()
                'Abjad 2.20 (development)'

        Returns string.
        '''
        result = 'Abjad {} ({})'
        result = result.format(
            class_.get_abjad_version_string(),
            'development',
            )
        return result

    @staticmethod
    def get_abjad_version_string():
        '''Gets Abjad version string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_abjad_version_string()
                '2.20'

        Returns string.
        '''
        import abjad
        return abjad.__version__

    @classmethod
    def get_lilypond_minimum_version_string(class_):
        r'''Gets LilyPond minimum version string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_lilypond_minimum_version_string() # doctest: +SKIP
                '2.17.0'

        Returns string.
        '''
        version = class_.get_lilypond_version_string()
        parts = version.split('.')[0:2]
        parts.append('0')
        return '.'.join(parts)

    @staticmethod
    def get_lilypond_version_string():
        '''Gets LilyPond version string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_lilypond_version_string() # doctest: +SKIP
                '2.19.1'

        Returns string.
        '''
        from abjad import abjad_configuration
        from abjad.tools import systemtools
        if AbjadConfiguration._lilypond_version_string is not None:
            return AbjadConfiguration._lilypond_version_string
        lilypond = abjad_configuration.get('lilypond_path')
        if not lilypond:
            lilypond = systemtools.IOManager.find_executable('lilypond')
            if lilypond:
                lilypond = lilypond[0]
            else:
                lilypond = 'lilypond'
        command = lilypond + ' --version'
        proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
        if sys.version_info[0] == 2:
            lilypond_version_string = proc.stdout.readline()
        else:
            import locale
            encoding = locale.getdefaultlocale()[1]
            if encoding is None:
                encoding = 'utf-8'
            lilypond_version_string = proc.stdout.readline().decode(encoding)
        lilypond_version_string = lilypond_version_string.split(' ')[-1]
        lilypond_version_string = lilypond_version_string.strip()
        AbjadConfiguration._lilypond_version_string = lilypond_version_string
        return lilypond_version_string

    @staticmethod
    def get_python_version_string():
        '''Gets Python version string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_python_version_string() # doctest: +SKIP
                '2.7.5'

        Returns string.
        '''
        return '.'.join(str(_) for _ in sys.version_info[:3])

    @staticmethod
    def get_tab_width():
        r'''Gets tab width.

        ..  container:: example

            ::

                >>> abjad_configuration.get_tab_width()
                4

        Used by code generation functions.

        Returns nonnegative integer.
        '''
        return 4

    @staticmethod
    def get_text_editor():
        r'''Gets text editor.

        ..  container:: example

            ::

                >>> abjad_configuration.get_text_editor() # doctest: +SKIP
                'vim'

        Returns string.
        '''
        from abjad import abjad_configuration
        text_editor = abjad_configuration['text_editor']
        if text_editor is not None:
            return text_editor
        elif os.name == 'posix':
            return 'vim'
        else:
            return 'edit'

    @staticmethod
    def list_package_dependency_versions():
        r'''Lists package dependency versions.

        ..  container:: example

            ::

                >>> abjad_configuration.list_package_dependency_versions() # doctest: +SKIP
                {'sphinx': '1.1.2', 'pytest': '2.1.2'}

        Returns dictionary.
        '''
        dependencies = {}
        dependencies['configobj'] = None
        try:
            import configobj
            dependencies['configobj'] = configobj.__version__
        except (AttributeError, ImportError):
            pass
        dependencies['ply'] = None
        try:
            from ply import lex
            dependencies['ply'] = lex.__version__
        except ImportError:
            pass
        dependencies['pytest'] = None
        try:
            import pytest
            dependencies['pytest'] = pytest.__version__
        except ImportError:
            pass
        dependencies['sphinx'] = None
        try:
            import sphinx
            dependencies['sphinx'] = sphinx.__version__
        except ImportError:
            pass
        return dependencies

    @staticmethod
    def set_default_accidental_spelling(spelling='mixed'):
        '''Sets default accidental spelling.

        ..  container:: example

            Sets default accidental spelling to sharps:

            ::

                >>> abjad_configuration.set_default_accidental_spelling('sharps')

            ::

                >>> [Note(13, (1, 4)), Note(15, (1, 4))]
                [Note("cs''4"), Note("ds''4")]

        ..  container:: example

            Sets default accidental spelling to flats:

            ::

                >>> abjad_configuration.set_default_accidental_spelling('flats')

            ::

                >>> [Note(13, (1, 4)), Note(15, (1, 4))]
                [Note("df''4"), Note("ef''4")]

        ..  container:: example

            Sets default accidental spelling to mixed:

            ::

                >>> abjad_configuration.set_default_accidental_spelling()

            ::

                >>> [Note(13, (1, 4)), Note(15, (1, 4))]
                [Note("cs''4"), Note("ef''4")]

        Defaults to ``'mixed'``.

        Mixed test case must appear last here for doc tests to check correctly.

        Returns none.
        '''
        from abjad import abjad_configuration
        if spelling not in ('mixed', 'sharps', 'flats'):
            raise ValueError
        abjad_configuration['accidental_spelling'] = spelling

    ### PRIVATE PROPERTIES ###

    @property
    def _initial_comment(self):
        current_time = self._current_time
        return [
            '-*- coding: utf-8 -*-',
            '',
            'Abjad configuration file created on {}.'.format(current_time),
            "This file is interpreted by Python's ConfigParser ",
            'and follows ini syntax.',
            ]

    ### PUBLIC PROPERTIES ###

    @property
    def abjad_boilerplate_directory(self):
        r'''Gest Abjad boilerplate directory.

        Return string.
        '''
        relative_path = os.path.join(
            self.abjad_directory,
            'boilerplate',
            )
        return os.path.abspath(relative_path)

    @property
    def abjad_directory(self):
        r'''Gets Abjad directory.

        Returns string.
        '''
        import abjad
        return abjad.__path__[0]

    @property
    def abjad_experimental_directory(self):
        r'''Gets Abjad experimental directory.

        Returns string.
        '''
        relative_path = os.path.join(
            self.abjad_root_directory,
            'experimental',
            )
        return os.path.abspath(relative_path)

    @property
    def abjad_output_directory(self):
        r'''Gets Abjad output directory.

        Returns string.
        '''
        if 'abjad_output_directory' in self._settings:
            return self._settings['abjad_output_directory']
        return os.path.join(
            self.configuration_directory_path,
            'output'
            )

    @property
    def abjad_root_directory(self):
        r'''Gets Abjad root directory.

        Returns string.
        '''
        relative_path = os.path.join(
            self.abjad_directory,
            '..',
            )
        return os.path.abspath(relative_path)

    @property
    def configuration_directory_name(self):
        r'''Gets configuration directory name.

        Returns string.
        '''
        return '.abjad'

    @property
    def configuration_file_name(self):
        r'''Gets configuration file name.

        Returns string.
        '''
        return 'abjad.cfg'

    @property
    def lilypond_log_file_path(self):
        r'''Gets LilyPond log file path.

        Returns string.
        '''
        return os.path.join(self.abjad_output_directory, 'lily.log')



# -*- coding: utf-8 -*-
import code


class AbjadBookConsole(code.InteractiveConsole):
    r'''An interactive console which provides a sandboxed namespace for
    executing abjad-book code examples.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Internals'

    ### INITIALIZER ###

    def __init__(
        self,
        document_handler=None,
        locals=None,
        filename='<stdin>',
        ):
        import abjad
        if locals is None:
            locals = abjad.__dict__.copy()
        locals['__builtins__'] = __builtins__.copy()
        locals['__name__'] = '__main__'
        locals['__package__'] = None
        code.InteractiveConsole.__init__(
            self,
            locals=locals,
            filename=filename,
            )
        self.document_handler = document_handler
        self.save_topleveltools_dict()
        self.push('from __future__ import print_function')

    ### PUBLIC METHODS ###

    def restore_topleveltools_dict(self):
        r'''Restores the topleveltols module dictionary.
        '''
        topleveltools = self.locals['topleveltools']
        topleveltools.__dict__.update(self.cached_topleveltools_dict)

    def save_topleveltools_dict(self):
        r'''Caches the dictionary of the topleveltools module.

        Because CodeBlock replaces various topleveltools function references
        with its own proxies, the originals must be cached so they can later be
        restored. Otherwise Sphinx's autodoc extension will discover the
        abjad-book function proxies and not the originals.
        '''
        topleveltools = self.locals['topleveltools']
        self.cached_topleveltools_dict = topleveltools.__dict__.copy()

    def showsyntaxerror(self, filename=None):
        r'''Proxies Python's InteractiveConsole.showsyntaxerror().
        '''
        code.InteractiveConsole.showsyntaxerror(self, filename=filename)
        self.document_handler.register_error()

    def showtraceback(self):
        r'''Proxies Python's InteractiveConsole.showtraceback().
        '''
        code.InteractiveConsole.showtraceback(self)
        self.document_handler.register_error()

    def unregister_error(self):
        r'''Unregisters the last error registered in the current document
        handler.

        This occurs when the interpreting code block permits errors to appear.
        '''
        if self.document_handler:
            self.document_handler.unregister_error()

    ### PUBLIC PROPERTIES ###

    @property
    def errored(self):
        r'''Is true if the last line executed by the console errored.
        '''
        if self.document_handler:
            return self.document_handler.errored
        return False



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import iterate


def get_previous_measure_from_component(component):
    '''Get previous measure from `component`.

    When `component` is voice, staff or other sequential context,
    and when `component` contains a measure, return last measure
    in `component`. This starts the process of backwards measure iteration.

    ::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
        >>> scoretools.get_previous_measure_from_component(staff)
        Measure((2, 8), "e'8 f'8")

    When `component` is voice, staff or other sequential context,
    and when `component` contains no measure,
    raise missing measure error.

    When `component` is a measure and there is a measure immediately
    preceeding `component`, return measure immediately preceeding component.

    ::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
        >>> scoretools.get_previous_measure_from_component(staff[-1])
        Measure((2, 8), "c'8 d'8")

    When `component` is a measure and there is no measure immediately
    preceeding `component`, return ``None``.

    ::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
        >>> scoretools.get_previous_measure_from_component(staff[0]) is None
        True

    When `component` is a leaf and there is a measure in the parentage
    of `component`, return the measure in the parentage of `component`.

    ::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
        >>> selector = select().by_leaf(flatten=True)
        >>> leaves = selector(staff)
        >>> scoretools.get_previous_measure_from_component(leaves[0])
        Measure((2, 8), "c'8 d'8")

    When `component` is a leaf and there is no measure in the parentage
    of `component`, raise missing measure error.
    '''
    from abjad.tools import scoretools

    if isinstance(component, scoretools.Leaf):
        for parent in component._get_parentage(include_self=False):
            if isinstance(parent, scoretools.Measure):
                return parent
        raise MissingMeasureError
    elif isinstance(component, scoretools.Measure):
        return component._get_in_my_logical_voice(
            -1, prototype=scoretools.Measure)
    elif isinstance(component, scoretools.Container):
        return scoretools.get_measure_that_stops_with_container(component)
    elif isinstance(component, (list, tuple)):
        measure_generator = iterate(component).by_class(
            scoretools.Measure, reverse=True)
        try:
            measure = next(measure_generator)
            return measure
        except StopIteration:
            raise MissingMeasureError
    else:
        message = 'unknown component: {!r}.'
        raise TypeError(message.format(component))


# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension_figured_bass_pair_01():

    #assert tonalanalysistools.ChordSuspension(9, 8).figured_bass_pair == (9, 8)
    assert tonalanalysistools.ChordSuspension(7, 6).figured_bass_pair == (7, 6)
    assert tonalanalysistools.ChordSuspension(4, 3).figured_bass_pair == (4, 3)
    assert tonalanalysistools.ChordSuspension(2, 1).figured_bass_pair == (2, 1)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Skip___setattr___01():
    r'''Slots constrain skip attributes.
    '''

    skip = scoretools.Skip((1, 4))

    assert pytest.raises(AttributeError, "skip.foo = 'bar'")



# -*- coding: utf-8 -*-
import pickle
from abjad import *


def test_quantizationtools_QuantizationJob_pickle_01():
    job_id = 1
    definition = {
        2: {
            2: {
                2: None
                },
                3: None
            },
            5: None
        }
    search_tree = quantizationtools.UnweightedSearchTree(definition)
    q_event_proxies = [
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent(0,      ['A'], index=1), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 5), ['B'], index=2), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 4), ['C'], index=3), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 3), ['D'], index=4), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 5), ['E'], index=5), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 2), ['F'], index=6), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 5), ['G'], index=7), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 3), ['H'], index=8), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 4), ['I'], index=9), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent((4, 5), ['J'], index=10), 0, 1),
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1,      ['K'], index=11), 0, 1)
        ]
    job = quantizationtools.QuantizationJob(job_id, search_tree, q_event_proxies)

    pickled = pickle.loads(pickle.dumps(job))

    assert pickled is not job
    assert format(pickled) == format(job), \
        systemtools.TestManager.diff(pickled, job, 'Diff:')

    job()

    pickled = pickle.loads(pickle.dumps(job))

    assert pickled is not job
    assert format(pickled) == format(job), \
        systemtools.TestManager.diff(pickled, job, 'Diff:')



# -*- coding: utf-8 -*-
import inspect
import pytest
from abjad.tools import abjadbooktools
from abjad.tools import datastructuretools
from abjad.tools import documentationtools
from abjad.tools import lilypondparsertools
from abjad.tools import rhythmtreetools
from abjad.tools import systemtools


ignored_names = (
    '__dict__',
    '__init__',
    '__new__',
    '__weakref__',
    'context_name',
    'denominator',
    'duration',
    'multiplier',
    'music',
    'numerator',
    'optional_id',
    'optional_context_mod',
    'push_signature',
    'type',
    'value',
    )

ignored_classes = (
    abjadbooktools.CodeBlock,
    abjadbooktools.LaTeXDocumentHandler,
    abjadbooktools.SphinxDocumentHandler,
    datastructuretools.Enumeration,
    lilypondparsertools.LilyPondLexicalDefinition,
    lilypondparsertools.LilyPondSyntacticalDefinition,
    lilypondparsertools.ReducedLyParser,
    lilypondparsertools.SchemeParser,
    rhythmtreetools.RhythmTreeParser,
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    systemtools.TestCase,
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.parametrize('obj', classes)
def test_abjad___doc___01(obj):
    r'''All classes have a docstring. All class methods have a docstring.
    '''
    missing_doc_names = []
    if obj.__doc__ is None:
        missing_doc_names.append(obj.__name__)
    for attr in inspect.classify_class_attrs(obj):
        if attr.name in ignored_names:
            continue
        elif attr.defining_class is not obj:
            continue
        if attr.name[0].isalpha() or attr.name.startswith('__'):
            if getattr(obj, attr.name).__doc__ is None:
                missing_doc_names.append(attr.name)
    if missing_doc_names:
        message = '\n'.join('{}.{}'.format(obj.__name__, name)
            for name in missing_doc_names)
        message = 'Missing docstrings for:\n{}'.format(message)
        raise Exception(message)


functions = documentationtools.list_all_abjad_functions()


@pytest.mark.parametrize('obj', functions)
def test_abjad___doc___02(obj):
    r'''All functions have a docstring.
    '''
    assert obj.__doc__ is not None



# -*- coding: utf-8 -*-


def cumulative_sums(argument, start=0):
    r'''Gets cumulative sums of `argument`.

    ..  container:: example

        ::

            >>> mathtools.cumulative_sums([1, 2, 3, 4, 5, 6, 7, 8], start=0)
            [0, 1, 3, 6, 10, 15, 21, 28, 36]

        ::

            >>> mathtools.cumulative_sums([1, 2, 3, 4, 5, 6, 7, 8], start=None)
            [1, 3, 6, 10, 15, 21, 28, 36]

    Raises exception when `argument` is not iterable.

    Returns new object of `argument` type.
    '''
    if start is None:
        result = []
    else:
        result = [start]
    for item in argument:
        if result:
            new = result[-1] + item
            result.append(new)
        else:
            result.append(item)
    return type(argument)(result)



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_TerminalQEvent___init___01():

    q_event = quantizationtools.TerminalQEvent(154)

    assert q_event.offset == durationtools.Offset(154)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_MutationAgent_rewrite_meter_01():

    source = parse('abj: | 4/4 8 2. 8 |')
    target = parse('abj: | 4/4 8 8 ~ 2 ~ 8 8 |')
    meter = metertools.Meter(source)
    mutate(source[:]).rewrite_meter(meter)
    assert format(source) == format(target)


def test_agenttools_MutationAgent_rewrite_meter_02():
    r'''Establishes meter when first component's score offset greater
    than zero.
    '''

    source = parse('abj: | 2/4 4 4 ~ || 4/4 8 2. 8 ~ || 2/4 4 4 |')
    target = parse('abj: | 2/4 4 4 ~ || 4/4 8 8 ~ 2 ~ 8 8 ~ || 2/4 4 4 |')
    meter = metertools.Meter(source[1])
    mutate(source[1][:]).rewrite_meter(meter)
    assert format(source) == format(target)


def test_agenttools_MutationAgent_rewrite_meter_03():
    r'''Descends into tuplets.
    '''

    source = parse('abj: | 2/4 2 ~ || 5/4 8 ~ 8 ~ 2/3 { 4 ~ 4 4 ~ }'
        '4 ~ 4 ~ || 2/4 2 |')
    target = parse('abj: | 2/4 2 ~ || 5/4 4 ~ 2/3 { 2 4 ~ } 2 ~ || 2/4 2 |')
    meter = metertools.Meter(source[1])
    mutate(source[1][:]).rewrite_meter(meter)
    assert format(source) == format(target)


def test_agenttools_MutationAgent_rewrite_meter_04():

    source = parse("abj: | 4/4 c'8. d'4.. e'4. |")
    target = parse("abj: | 4/4 c'8. d'16 ~ d'4. e'4. |")
    meter = metertools.Meter(source)
    mutate(source[:]).rewrite_meter(meter)
    assert format(source) == format(target)


def test_agenttools_MutationAgent_rewrite_meter_05():

    meter = metertools.Meter((4, 4))
    for rhythm_number in range(8):
        # without boundary enforcement
        notes = metertools.Meter._make_gridded_test_rhythm(
            4, rhythm_number, denominator=4)
        measure = Measure((4, 4), notes)
        mutate(measure[:]).rewrite_meter(meter)
        # with boundary enforcement
        notes = metertools.Meter._make_gridded_test_rhythm(
            4, rhythm_number, denominator=4)
        measure = Measure((4, 4), notes)
        mutate(measure[:]).rewrite_meter(meter, boundary_depth=-1)


def test_agenttools_MutationAgent_rewrite_meter_06():

    source = parse('abj: | 4/4 8 4. 2 |')
    target = parse('abj: | 4/4 8 4. 2 |')
    meter = metertools.Meter((4, 4))
    mutate(source[:]).rewrite_meter(meter)
    assert format(source) == format(target)


def test_agenttools_MutationAgent_rewrite_meter_07():
    r'''Can limit dot count.
    '''

    meter = '(4/4 (1/4 1/4 1/4 1/4))'

    maximum_dot_count = None
    source = parse('abj: | 4/4 2... 16 |')
    target = parse('abj: | 4/4 2... 16 |')
    mutate(source[:]).rewrite_meter(
        meter,
        maximum_dot_count=maximum_dot_count,
        )
    assert format(source) == format(target)

    maximum_dot_count = 3
    source = parse('abj: | 4/4 2... 16 |')
    target = parse('abj: | 4/4 2... 16 |')
    mutate(source[:]).rewrite_meter(
        meter,
        maximum_dot_count=maximum_dot_count,
        )
    assert format(source) == format(target)

    maximum_dot_count = 2
    source = parse('abj: | 4/4 2... 16 |')
    target = parse('abj: | 4/4 2. ~ 8. 16 |')
    mutate(source[:]).rewrite_meter(
        meter,
        maximum_dot_count=maximum_dot_count,
        )
    assert format(source) == format(target)

    maximum_dot_count = 1
    source = parse('abj: | 4/4 2... 16 |')
    target = parse('abj: | 4/4 2. ~ 8. 16 |')
    mutate(source[:]).rewrite_meter(
        meter,
        maximum_dot_count=maximum_dot_count,
        )
    assert format(source) == format(target)

    maximum_dot_count = 0
    source = parse('abj: | 4/4 2... 16 |')
    target = parse('abj: | 4/4 2 ~ 4 ~ 8 ~ 16 16 |')
    mutate(source[:]).rewrite_meter(
        meter,
        maximum_dot_count=maximum_dot_count,
        )
    assert format(source) == format(target)


def test_agenttools_MutationAgent_rewrite_meter_08():

    meter = metertools.Meter((4, 4))
    source = parse("abj: | 4/4 c'1 ~ || 4/4 4 ~ 4 ~ 4 ~ 4 ~ || 4/4 1 |")
    target = parse("abj: | 4/4 c'1 ~ || 4/4 1 ~ || 4/4 1 |")
    mutate(source[1]).rewrite_meter(meter)
    assert format(source) == format(target)



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_MutationAgent_scale_01():
    r'''Scales leaves by dot-generating multiplier.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    mutate(voice).scale(Multiplier(3, 2))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8.
            d'8.
            e'8.
            f'8.
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_scale_02():
    r'''Scales leaves by tie-generating multiplier.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    mutate(voice).scale(Multiplier(5, 4))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 ~
            c'32
            d'8 ~
            d'32
            e'8 ~
            e'32
            f'8 ~
            f'32
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_scale_03():
    r'''Scales leaves by tuplet-generating multiplier.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    mutate(voice).scale(Multiplier(4, 3))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'4
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                d'4
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                e'4
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                f'4
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_scale_04():
    r'''Scales leaves by tie- and tuplet-generating multiplier.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    mutate(voice).scale(Multiplier(5, 6))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'8 ~
                c'32
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                d'8 ~
                d'32
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                e'8 ~
                e'32
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                f'8 ~
                f'32
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_scale_05():
    r'''Scales mixed notes and tuplets.
    '''

    voice = Voice("c'8.")
    tuplet = scoretools.FixedDurationTuplet((3, 8), "d'8 e'8 f'8 g'8")
    voice.append(tuplet)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8.
            \tweak text #tuplet-number::calc-fraction-text
            \times 3/4 {
                d'8
                e'8
                f'8
                g'8
            }
        }
        '''
        )

    mutate(voice).scale(Multiplier(2, 3))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            {
                d'16
                e'16
                f'16
                g'16
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_scale_06():
    r'''Undo scale of 5/4 with scale of 4/5.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    mutate(voice).scale(Multiplier(5, 4))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 ~
            c'32
            d'8 ~
            d'32
            e'8 ~
            e'32
            f'8 ~
            f'32
        }
        '''
        )

    mutate(voice).scale(Multiplier(4, 5))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_scale_07():
    r'''Doubles measures.
    '''

    voice = Voice()
    voice.append(Measure((2, 8), "c'8 d'8"))
    voice.append(Measure((2, 8), "e'8 f'8"))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
        }
        '''
        )

    mutate(voice).scale(Multiplier(2))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/4
                c'4
                d'4
            }
            {
                e'4
                f'4
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_scale_08():
    r'''Scales measures by 5/4.
    '''

    voice = Voice()
    voice.append(Measure((2, 8), "c'8 d'8"))
    voice.append(Measure((2, 8), "e'8 f'8"))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
        }
        '''
        )

    mutate(voice).scale(Multiplier(5, 4))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 20/64
                c'8 ~
                c'32
                d'8 ~
                d'32
            }
            {
                e'8 ~
                e'32
                f'8 ~
                f'32
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_NoteRhythmMaker___call___01():

    maker = rhythmmakertools.NoteRhythmMaker()

    divisions = [(5, 16), (3, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/16
                c'4 ~
                c'16
            }
            {
                \time 3/8
                c'4.
            }
        }
        '''
        )


def test_rhythmmakertools_NoteRhythmMaker___call___02():

    duration_spelling_specifier = rhythmmakertools.DurationSpellingSpecifier(
        decrease_durations_monotonically=False,
        )
    maker = rhythmmakertools.NoteRhythmMaker(
        duration_spelling_specifier=duration_spelling_specifier,
        )

    divisions = [(5, 16), (3, 8)]
    leaf_lists = maker(divisions)
    leaves = Sequence(leaf_lists).flatten()

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/16
                c'16 ~
                c'4
            }
            {
                \time 3/8
                c'4.
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_01():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 2, 4)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/7 {
            c'16
            c'8
            c'4
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_02():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 1, 2, 4)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'16
            c'16
            c'8
            c'4
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_03():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((-2, 3, 7)),
        mathtools.NonreducedFraction(7, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 7/12 {
            r8
            c'8.
            c'4..
        }
        '''
        )

def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_04():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((7, 7, -4, -1)),
        mathtools.NonreducedFraction(1, 4),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 16/19 {
            c'16..
            c'16..
            r16
            r64
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_05():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 2, 2)),
        mathtools.NonreducedFraction(12, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'8
            c'4
            c'4
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_06():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((2, 4, 4)),
        mathtools.NonreducedFraction(12, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'8
            c'4
            c'4
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_07():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((4, 8, 8)),
        mathtools.NonreducedFraction(12, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/5 {
            c'4
            c'2
            c'2
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_08():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((8, 16, 16)),
        mathtools.NonreducedFraction(12, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/5 {
            c'4
            c'2
            c'2
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_09():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((2, 4, 4)),
        mathtools.NonreducedFraction(3, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/5 {
            c'16
            c'8
            c'8
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_10():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((2, 4, 4)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/5 {
            c'8
            c'4
            c'4
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_11():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((2, 4, 4)),
        mathtools.NonreducedFraction(12, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'8
            c'4
            c'4
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_12():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((2, 4, 4)),
        mathtools.NonreducedFraction(24, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'4
            c'2
            c'2
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_13():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 2, 2)),
        mathtools.NonreducedFraction(6, 2),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'2
            c'1
            c'1
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_14():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 2, 2)),
        mathtools.NonreducedFraction(6, 4),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'4
            c'2
            c'2
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_15():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 2, 2)),
        mathtools.NonreducedFraction(6, 8),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'8
            c'4
            c'4
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_16():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 2, 2)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/5 {
            c'16
            c'8
            c'8
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_17():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, -1, -1)),
        mathtools.NonreducedFraction(3, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            r16
            r16
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_18():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 1, -1, -1)),
        mathtools.NonreducedFraction(4, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            r16
            r16
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_19():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 1, 1, -1, -1)),
        mathtools.NonreducedFraction(5, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            c'16
            r16
            r16
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_20():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1, 1, 1, 1, -1, -1)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            c'16
            c'16
            r16
            r16
        }
        '''
        )


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_21():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((1,)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == 'Container("c\'4.")'


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_22():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((99,)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == 'Container("c\'4.")'


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_23():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((-1,)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "Container('r4.')"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_24():

    tuplet = Tuplet.from_nonreduced_ratio_and_nonreduced_fraction(
        mathtools.NonreducedRatio((-99,)),
        mathtools.NonreducedFraction(6, 16),
        )

    assert str(tuplet) == "Container('r4.')"


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_25():

    statement = 'Tuplet.from_nonreduced_ratio_and_nonreduced_fraction'
    statement += '([0], (3, 16))'
    pytest.raises(Exception, statement)


def test_scoretools_Tuplet_from_nonreduced_ratio_and_nonreduced_fraction_26():

    statement = 'Tuplet.from_nonreduced_ratio_and_nonreduced_fraction'
    statement += '([1, 1, 0, 1], (3, 16))'
    pytest.raises(Exception, statement)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.quantizationtools import *


def test_quantizationtools_QGrid_fit_q_events_01():

    q_grid = QGrid()

    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)

    q_grid.fit_q_events([a, b, c, d, e, f, g])

    assert q_grid.leaves[0].q_event_proxies == [a, b, c, d]
    assert q_grid.leaves[1].q_event_proxies == [e, f, g]

    q_events = q_grid.subdivide_leaves([(0, (1, 1))])

    q_grid.fit_q_events(q_events)

    assert q_grid.leaves[0].q_event_proxies == [a, b]
    assert q_grid.leaves[1].q_event_proxies == [c, d, e]
    assert q_grid.leaves[2].q_event_proxies == [g, f]



# -*- coding: utf-8 -*-


def all_are_equal(argument):
    '''Is true when `argument` is an iterable collection of equal items.
    Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.all_are_equal([99, 99, 99, 99, 99, 99])
            True

        ::

            >>> mathtools.all_are_equal(17)
            False

    ..  container:: example

        Is true when `argument` is empty:

        ::

            >>> mathtools.all_are_equal([])
            True

    Returns true or false.
    '''
    try:
        first_element = None
        for element in argument:
            if first_element is None:
                first_element = element
            else:
                if not element == first_element:
                    return False
        return True
    except TypeError:
        return False



# -*- coding: utf-8 -*-
from abjad.tools.quantizationtools.GraceHandler import GraceHandler


class DiscardingGraceHandler(GraceHandler):
    r'''Concrete ``GraceHandler`` subclass which discards all but final
    ``QEvent`` attached to an offset.

    Does not create ``GraceContainers``.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, q_events):
        r'''Calls idscarind grace handler.
        '''
        from abjad.tools import quantizationtools
        q_event = q_events[-1]
        if isinstance(q_event, quantizationtools.PitchedQEvent):
            return q_event.pitches, None
        return (), None



# -*- coding: utf-8 -*-


def integer_to_binary_string(n):
    r'''Changes positive integer `n` to binary string.

    ..  container:: example

        ::

            >>> for n in range(1, 16 + 1):
            ...     print('{}\t{}'.format(n, mathtools.integer_to_binary_string(n)))
            ... 
            1  1
            2  10
            3  11
            4  100
            5  101
            6  110
            7  111
            8  1000
            9  1001
            10 1010
            11 1011
            12 1100
            13 1101
            14 1110
            15 1111
            16 10000

    Returns string.
    '''
    if n == 0:
        return '0'
    result = bin(abs(n)).lstrip('-0b')
    if n < 0:
        result = '-' + result
    return result



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Instrument import Instrument


class Glockenspiel(Instrument):
    r'''A glockenspiel.

    ::

        >>> staff = Staff("c'4 d'4 e'4 fs'4")
        >>> glockenspiel = instrumenttools.Glockenspiel()
        >>> attach(glockenspiel, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Glockenspiel }
            \set Staff.shortInstrumentName = \markup { Gkspl. }
            c'4
            d'4
            e'4
            fs'4
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='glockenspiel',
        short_instrument_name='gkspl.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=None,
        pitch_range='[G5, C8]',
        sounding_pitch_of_written_middle_c='C6',
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._performer_names.extend([
            'percussionist',
            ])

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets glockenspiel's allowable clefs.

        ..  container:: example

            ::

                >>> glockenspiel.allowable_clefs
                ClefList([Clef(name='treble')])

            ::

                >>> show(glockenspiel.allowable_clefs) # doctest: +SKIP

        Returns clef list.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets glockenspiel's name.

        ..  container:: example

            ::

                >>> glockenspiel.instrument_name
                'glockenspiel'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets glockenspiel's instrument name markup.

        ..  container:: example

            ::

                >>> glockenspiel.instrument_name_markup
                Markup(contents=['Glockenspiel'])

            ::

                >>> show(glockenspiel.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets glockenspiel's range.

        ..  container:: example

            ::

                >>> glockenspiel.pitch_range
                PitchRange(range_string='[G5, C8]')

            ::

                >>> show(glockenspiel.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets glockenspiel's short instrument name.

        ..  container:: example

            ::

                >>> glockenspiel.short_instrument_name
                'gkspl.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets glockenspiel's short instrument name markup.

        ..  container:: example

            ::

                >>> glockenspiel.short_instrument_name_markup
                Markup(contents=['Gkspl.'])

            ::

                >>> show(glockenspiel.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r"""Gets sounding pitch of glockenspiel's written middle C.

        ..  container:: example

            ::

                >>> glockenspiel.sounding_pitch_of_written_middle_c
                NamedPitch("c'''")

            ::

                >>> show(glockenspiel.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        """
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeContainer_pop_01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()

    container = datastructuretools.TreeContainer([leaf_a, leaf_b, leaf_c])
    assert container.children == (leaf_a, leaf_b, leaf_c)
    assert leaf_a.parent is container
    assert leaf_b.parent is container
    assert leaf_c.parent is container

    result = container.pop()
    assert container.children == (leaf_a, leaf_b)
    assert result is leaf_c
    assert result.parent is None

    result = container.pop(0)
    assert container.children == (leaf_b,)
    assert result is leaf_a
    assert result.parent is None



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment_retrograde_01():

    pitch_class_segment_1 = pitchtools.PitchClassSegment([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),
        pitchtools.NamedPitchClass('f'),
        pitchtools.NamedPitchClass('g'),])

    pitch_class_segment_2 = pitchtools.PitchClassSegment([
        pitchtools.NamedPitchClass('g'),
        pitchtools.NamedPitchClass('f'),
        pitchtools.NamedPitchClass('e'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('c'),])

    assert pitch_class_segment_1.retrograde() == pitch_class_segment_2
    assert pitch_class_segment_2.retrograde() == pitch_class_segment_1


def test_pitchtools_PitchClassSegment_retrograde_02():

    pitch_class_segment= pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])

    assert pitch_class_segment.retrograde() == \
        pitchtools.PitchClassSegment([2, 9, 4, 10, 6, 0])



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadList___len___01():

    assert len(Chord('<>4').note_heads) == 0
    assert len(Chord("<ef'>4").note_heads) == 1
    assert len(Chord("<ef' cs''>4").note_heads) == 2
    assert len(Chord("<ef' cs'' f''>4").note_heads) == 3



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_topmost_logical_ties_and_components_01():
    r'''Iterates toplevel contents with logical ties in place of leaves.
    '''

    staff = Staff(r"c'8 ~ c'32 g'8 ~ g'32 a'8 ~ a'32 b'8 ~ b'32")
    tuplet = scoretools.FixedDurationTuplet((2, 8), "c'8 d'8 e'8")
    staff.insert(4, tuplet)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 ~
            c'32
            g'8 ~
            g'32
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            a'8 ~
            a'32
            b'8 ~
            b'32
        }
        '''
        )

    contents = iterate(staff).by_topmost_logical_ties_and_components()
    contents = list(contents)

    assert contents[0] == inspect_(staff[0]).get_logical_tie()
    assert contents[1] == inspect_(staff[2]).get_logical_tie()
    assert contents[2] is staff[4]
    assert contents[3] == inspect_(staff[5]).get_logical_tie()
    assert contents[4] == inspect_(staff[7]).get_logical_tie()



# -*- coding: utf-8 -*-
import numbers
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import select


class ItemSelectorCallback(AbjadValueObject):
    r'''Item selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_apply_to_each',
        '_item',
        )

    ### INITIALIZER ###

    def __init__(self, item=0, apply_to_each=True):
        assert isinstance(item, int)
        self._item = item
        self._apply_to_each = bool(apply_to_each)

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Gets item from `argument`.

        Returns item.
        '''
        import abjad
        assert isinstance(argument, (abjad.Selection, tuple)), repr(argument)
        if self.apply_to_each:
            result = []
            for item in argument:
                result_ = self._get_item(item)
                result.append(result_)
            result = tuple(result)
        else:
            result = self._get_item(argument)
        return result

    ### PRIVATE METHODS ###

    def _get_item(self, argument):
        result = argument.__getitem__(self.item)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def apply_to_each(self):
        r'''Is true if item selector callback will be applied against the
        contents of each selection, rather than against the sequence of
        selections itself.

        Otherwise false.

        Returns true or false.
        '''
        return self._apply_to_each

    @property
    def item(self):
        r'''Gets item selector callback item.

        Returns integer.
        '''
        return self._item



# -*- coding: utf-8 -*-
import abc
import collections
from abjad.tools import indicatortools
from abjad.tools.abctools import AbjadObject


class QSchemaItem(AbjadObject):
    '''`QSchemaItem` represents a change of state in the timeline of a
    quantization process.

    `QSchemaItem` is abstract and immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_search_tree',
        '_tempo',
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(
        self,
        search_tree=None,
        tempo=None,
        ):
        from abjad.tools import quantizationtools
        if search_tree is not None:
            assert isinstance(search_tree, quantizationtools.SearchTree)
        self._search_tree = search_tree
        #if tempo is not None:
        #    tempo = indicatortools.Tempo(tempo)
        #    assert not tempo.is_imprecise
        if tempo is not None:
            if isinstance(tempo, tuple):
                tempo = indicatortools.Tempo(*tempo)
            assert not tempo.is_imprecise
        self._tempo = tempo

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats q schema item.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    ### PUBLIC PROPERTIES ###

    @property
    def search_tree(self):
        r'''The optionally defined search tree.

        Returns search tree or none.
        '''
        return self._search_tree

    @property
    def tempo(self):
        r'''The optionally defined tempo.

        Returns tempo or none.
        '''
        return self._tempo



# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadObject


class Infinity(AbjadObject):
    r'''Infinity.

    ..  container:: example

        All numbers compare less than infinity:

        ::

            >>> 9999999 < Infinity
            True

        ::

            >>> 2**38 < Infinity
            True

    ..  container:: example

        Infinity compares equal to itself:

        ::

            >>> Infinity == Infinity
            True

    ..  container:: example

        Negative infinity compares less than infinity:

        ::

            >>> NegativeInfinity < Infinity
            True
    
    Initializes as a system singleton at start-up.

    Available as a built-in after Abjad starts.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_value',
        )

    ### INTIALIZER ###

    def __init__(self):
        self._value = float('infinity')

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is also infinity. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self._value == argument._value
        return False

    def __float__(self):
        r'''Convert infinity to float.

        Returns float.
        '''
        return self._value

    def __ge__(self, argument):
        r'''Is true for all values of `argument`.

        Returns true.
        '''
        return self._value >= argument

    def __gt__(self, argument):
        r'''Is true for all noninfinite values of `argument`. Otherwise false.

        Returns true or false.
        '''
        return self._value > argument

    def __hash__(self):
        r'''Hashes infinity.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Infinity, self).__hash__()

    def __le__(self, argument):
        r'''Is true when `argument` is infinite. Otherwise false.

        Returns true or false.
        '''
        return self._value <= argument

    def __lt__(self, argument):
        r'''Is true for no values of `argument`.

        Returns true or false.
        '''
        return self._value < argument

    def __sub__(self, argument):
        r'''Subtracts `argument` from infinity.

        Returns infinity or 0 if `argument` is also infinity.
        '''
        if argument is self:
            return 0
        return self

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_text=type(self).__name__,
            storage_format_text=type(self).__name__,
            )
